class TimeZoneInfo(object,IEquatable[TimeZoneInfo],ISerializable,IDeserializationCallback):
 """ Represents any time zone in the world. """
 @staticmethod
 def ClearCachedData():
  """
  ClearCachedData()

   Clears cached time zone data.
  """
  pass
 @staticmethod
 def ConvertTime(*__args):
  """
  ConvertTime(dateTime: DateTime,sourceTimeZone: TimeZoneInfo,destinationTimeZone: TimeZoneInfo) -> DateTime

  

   Converts a time from one time zone to another.

  

   dateTime: The date and time to convert.

   sourceTimeZone: The time zone of dateTime.

   destinationTimeZone: The time zone to convert dateTime to.

   Returns: The date and time in the destination time zone that corresponds to the dateTime parameter in the 

    source time zone.

  

  ConvertTime(dateTime: DateTime,destinationTimeZone: TimeZoneInfo) -> DateTime

  

   Converts a time to the time in a particular time zone.

  

   dateTime: The date and time to convert.

   destinationTimeZone: The time zone to convert dateTime to.

   Returns: The date and time in the destination time zone.

  ConvertTime(dateTimeOffset: DateTimeOffset,destinationTimeZone: TimeZoneInfo) -> DateTimeOffset

  

   Converts a time to the time in a particular time zone.

  

   dateTimeOffset: The date and time to convert.

   destinationTimeZone: The time zone to convert dateTime to.

   Returns: The date and time in the destination time zone.
  """
  pass
 @staticmethod
 def ConvertTimeBySystemTimeZoneId(*__args):
  """
  ConvertTimeBySystemTimeZoneId(dateTime: DateTime,sourceTimeZoneId: str,destinationTimeZoneId: str) -> DateTime

  

   Converts a time from one time zone to another based on time zone identifiers.

  

   dateTime: The date and time to convert.

   sourceTimeZoneId: The identifier of the source time zone.

   destinationTimeZoneId: The identifier of the destination time zone.

   Returns: The date and time in the destination time zone that corresponds to the dateTime parameter in the 

    source time zone.

  

  ConvertTimeBySystemTimeZoneId(dateTime: DateTime,destinationTimeZoneId: str) -> DateTime

  

   Converts a time to the time in another time zone based on the time zone's identifier.

  

   dateTime: The date and time to convert.

   destinationTimeZoneId: The identifier of the destination time zone.

   Returns: The date and time in the destination time zone.

  ConvertTimeBySystemTimeZoneId(dateTimeOffset: DateTimeOffset,destinationTimeZoneId: str) -> DateTimeOffset

  

   Converts a time to the time in another time zone based on the time zone's identifier.

  

   dateTimeOffset: The date and time to convert.

   destinationTimeZoneId: The identifier of the destination time zone.

   Returns: The date and time in the destination time zone.
  """
  pass
 @staticmethod
 def ConvertTimeFromUtc(dateTime,destinationTimeZone):
  """
  ConvertTimeFromUtc(dateTime: DateTime,destinationTimeZone: TimeZoneInfo) -> DateTime

  

   Converts a Coordinated Universal Time (UTC) to the time in a specified time zone.

  

   dateTime: The Coordinated Universal Time (UTC).

   destinationTimeZone: The time zone to convert dateTime to.

   Returns: The date and time in the destination time zone. Its System.DateTime.Kind property is 

    System.DateTimeKind.Utc if destinationTimeZone is System.TimeZoneInfo.Utc; otherwise,its 

    System.DateTime.Kind property is System.DateTimeKind.Unspecified.
  """
  pass
 @staticmethod
 def ConvertTimeToUtc(dateTime,sourceTimeZone=None):
  """
  ConvertTimeToUtc(dateTime: DateTime,sourceTimeZone: TimeZoneInfo) -> DateTime

  

   Converts the time in a specified time zone to Coordinated Universal Time (UTC).

  

   dateTime: The date and time to convert.

   sourceTimeZone: The time zone of dateTime.

   Returns: The Coordinated Universal Time (UTC) that corresponds to the dateTime parameter. The 

    System.DateTime object's System.DateTime.Kind property is always set to System.DateTimeKind.Utc.

  

  ConvertTimeToUtc(dateTime: DateTime) -> DateTime

  

   Converts the current date and time to Coordinated Universal Time (UTC).

  

   dateTime: The date and time to convert.

   Returns: The Coordinated Universal Time (UTC) that corresponds to the dateTime parameter. The 

    System.DateTime value's System.DateTime.Kind property is always set to System.DateTimeKind.Utc.
  """
  pass
 @staticmethod
 def CreateCustomTimeZone(id,baseUtcOffset,displayName,standardDisplayName,daylightDisplayName=None,adjustmentRules=None,disableDaylightSavingTime=None):
  """
  CreateCustomTimeZone(id: str,baseUtcOffset: TimeSpan,displayName: str,standardDisplayName: str,daylightDisplayName: str,adjustmentRules: Array[AdjustmentRule],disableDaylightSavingTime: bool) -> TimeZoneInfo

  CreateCustomTimeZone(id: str,baseUtcOffset: TimeSpan,displayName: str,standardDisplayName: str,daylightDisplayName: str,adjustmentRules: Array[AdjustmentRule]) -> TimeZoneInfo

  CreateCustomTimeZone(id: str,baseUtcOffset: TimeSpan,displayName: str,standardDisplayName: str) -> TimeZoneInfo

  

   Creates a custom time zone with a specified identifier,an offset from Coordinated Universal 

    Time (UTC),a display name,and a standard time display name.

  

  

   id: The time zone's identifier.

   baseUtcOffset: An object that represents the time difference between this time zone and Coordinated Universal 

    Time (UTC).

  

   displayName: The display name of the new time zone.

   standardDisplayName: The name of the new time zone's standard time.

   Returns: The new time zone.
  """
  pass
 def Equals(self,*__args):
  """
  Equals(self: TimeZoneInfo,obj: object) -> bool

  Equals(self: TimeZoneInfo,other: TimeZoneInfo) -> bool

  

   Determines whether the current System.TimeZoneInfo object and another System.TimeZoneInfo object 

    are equal.

  

  

   other: A second object to compare with the current object.

   Returns: true if the two System.TimeZoneInfo objects are equal; otherwise,false.
  """
  pass
 @staticmethod
 def FindSystemTimeZoneById(id):
  """
  FindSystemTimeZoneById(id: str) -> TimeZoneInfo

  

   Retrieves a System.TimeZoneInfo object from the registry based on its identifier.

  

   id: The time zone identifier,which corresponds to the System.TimeZoneInfo.Id property.

   Returns: An object whose identifier is the value of the id parameter.
  """
  pass
 @staticmethod
 def FromSerializedString(source):
  """
  FromSerializedString(source: str) -> TimeZoneInfo

  

   Deserializes a string to re-create an original serialized System.TimeZoneInfo object.

  

   source: The string representation of the serialized System.TimeZoneInfo object.

   Returns: The original serialized object.
  """
  pass
 def GetAdjustmentRules(self):
  """
  GetAdjustmentRules(self: TimeZoneInfo) -> Array[AdjustmentRule]

  

   Retrieves an array of System.TimeZoneInfo.AdjustmentRule objects that apply to the current 

    System.TimeZoneInfo object.

  

   Returns: An array of objects for this time zone.
  """
  pass
 def GetAmbiguousTimeOffsets(self,*__args):
  """
  GetAmbiguousTimeOffsets(self: TimeZoneInfo,dateTime: DateTime) -> Array[TimeSpan]

  

   Returns information about the possible dates and times that an ambiguous date and time can be 

    mapped to.

  

  

   dateTime: A date and time.

   Returns: An array of objects that represents possible Coordinated Universal Time (UTC) offsets that a 

    particular date and time can be mapped to.

  

  GetAmbiguousTimeOffsets(self: TimeZoneInfo,dateTimeOffset: DateTimeOffset) -> Array[TimeSpan]

  

   Returns information about the possible dates and times that an ambiguous date and time can be 

    mapped to.

  

  

   dateTimeOffset: A date and time.

   Returns: An array of objects that represents possible Coordinated Universal Time (UTC) offsets that a 

    particular date and time can be mapped to.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: TimeZoneInfo) -> int

  

   Serves as a hash function for hashing algorithms and data structures such as hash tables.

   Returns: A 32-bit signed integer that serves as the hash code for this System.TimeZoneInfo object.
  """
  pass
 @staticmethod
 def GetSystemTimeZones():
  """
  GetSystemTimeZones() -> ReadOnlyCollection[TimeZoneInfo]

  

   Returns a sorted collection of all the time zones about which information is available on the 

    local system.

  

   Returns: A read-only collection of System.TimeZoneInfo objects.
  """
  pass
 def GetUtcOffset(self,*__args):
  """
  GetUtcOffset(self: TimeZoneInfo,dateTime: DateTime) -> TimeSpan

  

   Calculates the offset or difference between the time in this time zone and Coordinated Universal 

    Time (UTC) for a particular date and time.

  

  

   dateTime: The date and time to determine the offset for.

   Returns: An object that indicates the time difference between the two time zones.

  GetUtcOffset(self: TimeZoneInfo,dateTimeOffset: DateTimeOffset) -> TimeSpan

  

   Calculates the offset or difference between the time in this time zone and Coordinated Universal 

    Time (UTC) for a particular date and time.

  

  

   dateTimeOffset: The date and time to determine the offset for.

   Returns: An object that indicates the time difference between Coordinated Universal Time (UTC) and the 

    current time zone.
  """
  pass
 def HasSameRules(self,other):
  """
  HasSameRules(self: TimeZoneInfo,other: TimeZoneInfo) -> bool

  

   Indicates whether the current object and another System.TimeZoneInfo object have the same 

    adjustment rules.

  

  

   other: A second object to compare with the current System.TimeZoneInfo object.

   Returns: true if the two time zones have identical adjustment rules and an identical base offset; 

    otherwise,false.
  """
  pass
 def IsAmbiguousTime(self,*__args):
  """
  IsAmbiguousTime(self: TimeZoneInfo,dateTime: DateTime) -> bool

  

   Determines whether a particular date and time in a particular time zone is ambiguous and can be 

    mapped to two or more Coordinated Universal Time (UTC) times.

  

  

   dateTime: A date and time value.

   Returns: true if the dateTime parameter is ambiguous; otherwise,false.

  IsAmbiguousTime(self: TimeZoneInfo,dateTimeOffset: DateTimeOffset) -> bool

  

   Determines whether a particular date and time in a particular time zone is ambiguous and can be 

    mapped to two or more Coordinated Universal Time (UTC) times.

  

  

   dateTimeOffset: A date and time.

   Returns: true if the dateTimeOffset parameter is ambiguous in the current time zone; otherwise,false.
  """
  pass
 def IsDaylightSavingTime(self,*__args):
  """
  IsDaylightSavingTime(self: TimeZoneInfo,dateTime: DateTime) -> bool

  

   Indicates whether a specified date and time falls in the range of daylight saving time for the 

    time zone of the current System.TimeZoneInfo object.

  

  

   dateTime: A date and time value.

   Returns: true if the dateTime parameter is a daylight saving time; otherwise,false.

  IsDaylightSavingTime(self: TimeZoneInfo,dateTimeOffset: DateTimeOffset) -> bool

  

   Indicates whether a specified date and time falls in the range of daylight saving time for the 

    time zone of the current System.TimeZoneInfo object.

  

  

   dateTimeOffset: A date and time value.

   Returns: true if the dateTimeOffset parameter is a daylight saving time; otherwise,false.
  """
  pass
 def IsInvalidTime(self,dateTime):
  """
  IsInvalidTime(self: TimeZoneInfo,dateTime: DateTime) -> bool

  

   Indicates whether a particular date and time is invalid.

  

   dateTime: A date and time value.

   Returns: true if dateTime is invalid; otherwise,false.
  """
  pass
 def ToSerializedString(self):
  """
  ToSerializedString(self: TimeZoneInfo) -> str

  

   Converts the current System.TimeZoneInfo object to a serialized string.

   Returns: A string that represents the current System.TimeZoneInfo object.
  """
  pass
 def ToString(self):
  """
  ToString(self: TimeZoneInfo) -> str

  

   Returns the current System.TimeZoneInfo object's display name.

   Returns: The value of the System.TimeZoneInfo.DisplayName property of the current System.TimeZoneInfo 

    object.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 def __str__(self,*args):
  pass
 BaseUtcOffset=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the time difference between the current time zone's standard time and Coordinated Universal Time (UTC).



Get: BaseUtcOffset(self: TimeZoneInfo) -> TimeSpan



"""

 DaylightName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the localized display name for the current time zone's daylight saving time.



Get: DaylightName(self: TimeZoneInfo) -> str



"""

 DisplayName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the localized general display name that represents the time zone.



Get: DisplayName(self: TimeZoneInfo) -> str



"""

 Id=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the time zone identifier.



Get: Id(self: TimeZoneInfo) -> str



"""

 StandardName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the localized display name for the time zone's standard time.



Get: StandardName(self: TimeZoneInfo) -> str



"""

 SupportsDaylightSavingTime=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the time zone has any daylight saving time rules.



Get: SupportsDaylightSavingTime(self: TimeZoneInfo) -> bool



"""


 AdjustmentRule=None
 Local=None
 TransitionTime=None
 Utc=None




class STATSTG(object):
 """ Use System.Runtime.InteropServices.ComTypes.STATSTG instead. """
 atime=None
 cbSize=None
 clsid=None
 ctime=None
 grfLocksSupported=None
 grfMode=None
 grfStateBits=None
 mtime=None
 pwcsName=None
 reserved=None
 type=None




class LicenseProvider(object):
 """ Provides the abstract base class for implementing a license provider. """
 def GetLicense(self,context,type,instance,allowExceptions):
  """
  GetLicense(self: LicenseProvider,context: LicenseContext,type: Type,instance: object,allowExceptions: bool) -> License

  

   When overridden in a derived class,gets a license for an instance or type of component,when 

    given a context and whether the denial of a license throws an exception.

  

  

   context: A System.ComponentModel.LicenseContext that specifies where you can use the licensed object.

   type: A System.Type that represents the component requesting the license.

   instance: An object that is requesting the license.

   allowExceptions: true if a System.ComponentModel.LicenseException should be thrown when the component cannot be 

    granted a license; otherwise,false.

  

   Returns: A valid System.ComponentModel.License.
  """
  pass



class DataGridViewRowCollection(object,ICollection,IEnumerable,IList):
 """
 A collection of System.Windows.Forms.DataGridViewRow objects.

 

 DataGridViewRowCollection(dataGridView: DataGridView)
 """
 def Add(self,*__args):
  """
  Add(self: DataGridViewRowCollection,dataGridViewRow: DataGridViewRow) -> int

  

   Adds the specified System.Windows.Forms.DataGridViewRow to the collection.

  

   dataGridViewRow: The System.Windows.Forms.DataGridViewRow to add to the 

    System.Windows.Forms.DataGridViewRowCollection.

  

   Returns: The index of the new System.Windows.Forms.DataGridViewRow.

  Add(self: DataGridViewRowCollection,count: int) -> int

  

   Adds the specified number of new rows to the collection.

  

   count: The number of rows to add to the System.Windows.Forms.DataGridViewRowCollection.

   Returns: The index of the last row that was added.

  Add(self: DataGridViewRowCollection) -> int

  

   Adds a new row to the collection.

   Returns: The index of the new row.

  Add(self: DataGridViewRowCollection,*values: Array[object]) -> int

  

   Adds a new row to the collection,and populates the cells with the specified objects.

  

   values: A variable number of objects that populate the cells of the new 

    System.Windows.Forms.DataGridViewRow.

  

   Returns: The index of the new row.
  """
  pass
 def AddCopies(self,indexSource,count):
  """
  AddCopies(self: DataGridViewRowCollection,indexSource: int,count: int) -> int

  

   Adds the specified number of rows to the collection based on the row at the specified index.

  

   indexSource: The index of the row on which to base the new rows.

   count: The number of rows to add to the System.Windows.Forms.DataGridViewRowCollection.

   Returns: The index of the last row that was added.
  """
  pass
 def AddCopy(self,indexSource):
  """
  AddCopy(self: DataGridViewRowCollection,indexSource: int) -> int

  

   Adds a new row based on the row at the specified index.

  

   indexSource: The index of the row on which to base the new row.

   Returns: The index of the new row.
  """
  pass
 def AddRange(self,dataGridViewRows):
  """
  AddRange(self: DataGridViewRowCollection,*dataGridViewRows: Array[DataGridViewRow])

   Adds the specified System.Windows.Forms.DataGridViewRow objects to the collection.

  

   dataGridViewRows: An array of System.Windows.Forms.DataGridViewRow objects to be added to the 

    System.Windows.Forms.DataGridViewRowCollection.
  """
  pass
 def Clear(self):
  """
  Clear(self: DataGridViewRowCollection)

   Clears the collection.
  """
  pass
 def Contains(self,dataGridViewRow):
  """
  Contains(self: DataGridViewRowCollection,dataGridViewRow: DataGridViewRow) -> bool

  

   Determines whether the specified System.Windows.Forms.DataGridViewRow is in the collection.

  

   dataGridViewRow: The System.Windows.Forms.DataGridViewRow to locate in the 

    System.Windows.Forms.DataGridViewRowCollection.

  

   Returns: true if the System.Windows.Forms.DataGridViewRow is in the 

    System.Windows.Forms.DataGridViewRowCollection; otherwise,false.
  """
  pass
 def CopyTo(self,array,index):
  """
  CopyTo(self: DataGridViewRowCollection,array: Array[DataGridViewRow],index: int)

   Copies the items from the collection into the specified System.Windows.Forms.DataGridViewRow 

    array,starting at the specified index.

  

  

   array: A System.Windows.Forms.DataGridViewRow array that is the destination of the items copied from 

    the System.Windows.Forms.DataGridViewRowCollection.

  

   index: The zero-based index in array at which copying begins.
  """
  pass
 def GetFirstRow(self,includeFilter,excludeFilter=None):
  """
  GetFirstRow(self: DataGridViewRowCollection,includeFilter: DataGridViewElementStates,excludeFilter: DataGridViewElementStates) -> int

  

   Returns the index of the first System.Windows.Forms.DataGridViewRow that meets the specified 

    inclusion and exclusion criteria.

  

  

   includeFilter: A bitwise combination of System.Windows.Forms.DataGridViewElementStates values.

   excludeFilter: A bitwise combination of System.Windows.Forms.DataGridViewElementStates values.

   Returns: The index of the first System.Windows.Forms.DataGridViewRow that has the attributes specified by 

    includeFilter,and does not have the attributes specified by excludeFilter; -1 if no row is 

    found.

  

  GetFirstRow(self: DataGridViewRowCollection,includeFilter: DataGridViewElementStates) -> int

  

   Returns the index of the first System.Windows.Forms.DataGridViewRow that meets the specified 

    criteria.

  

  

   includeFilter: A bitwise combination of System.Windows.Forms.DataGridViewElementStates values.

   Returns: The index of the first System.Windows.Forms.DataGridViewRow that has the attributes specified by 

    includeFilter; -1 if no row is found.
  """
  pass
 def GetLastRow(self,includeFilter):
  """
  GetLastRow(self: DataGridViewRowCollection,includeFilter: DataGridViewElementStates) -> int

  

   Returns the index of the last System.Windows.Forms.DataGridViewRow that meets the specified 

    criteria.

  

  

   includeFilter: A bitwise combination of System.Windows.Forms.DataGridViewElementStates values.

   Returns: The index of the last System.Windows.Forms.DataGridViewRow that has the attributes specified by 

    includeFilter; -1 if no row is found.
  """
  pass
 def GetNextRow(self,indexStart,includeFilter,excludeFilter=None):
  """
  GetNextRow(self: DataGridViewRowCollection,indexStart: int,includeFilter: DataGridViewElementStates,excludeFilter: DataGridViewElementStates) -> int

  

   Returns the index of the next System.Windows.Forms.DataGridViewRow that meets the specified 

    inclusion and exclusion criteria.

  

  

   indexStart: The index of the row where the method should begin to look for the next 

    System.Windows.Forms.DataGridViewRow.

  

   includeFilter: A bitwise combination of System.Windows.Forms.DataGridViewElementStates values.

   excludeFilter: A bitwise combination of System.Windows.Forms.DataGridViewElementStates values.

   Returns: The index of the next System.Windows.Forms.DataGridViewRow that has the attributes specified by 

    includeFilter,and does not have the attributes specified by excludeFilter; -1 if no row is 

    found.

  

  GetNextRow(self: DataGridViewRowCollection,indexStart: int,includeFilter: DataGridViewElementStates) -> int

  

   Returns the index of the next System.Windows.Forms.DataGridViewRow that meets the specified 

    criteria.

  

  

   indexStart: The index of the row where the method should begin to look for the next 

    System.Windows.Forms.DataGridViewRow.

  

   includeFilter: A bitwise combination of System.Windows.Forms.DataGridViewElementStates values.

   Returns: The index of the first System.Windows.Forms.DataGridViewRow after indexStart that has the 

    attributes specified by includeFilter,or -1 if no row is found.
  """
  pass
 def GetPreviousRow(self,indexStart,includeFilter,excludeFilter=None):
  """
  GetPreviousRow(self: DataGridViewRowCollection,indexStart: int,includeFilter: DataGridViewElementStates,excludeFilter: DataGridViewElementStates) -> int

  

   Returns the index of the previous System.Windows.Forms.DataGridViewRow that meets the specified 

    inclusion and exclusion criteria.

  

  

   indexStart: The index of the row where the method should begin to look for the previous 

    System.Windows.Forms.DataGridViewRow.

  

   includeFilter: A bitwise combination of System.Windows.Forms.DataGridViewElementStates values.

   excludeFilter: A bitwise combination of System.Windows.Forms.DataGridViewElementStates values.

   Returns: The index of the previous System.Windows.Forms.DataGridViewRow that has the attributes specified 

    by includeFilter,and does not have the attributes specified by excludeFilter; -1 if no row is 

    found.

  

  GetPreviousRow(self: DataGridViewRowCollection,indexStart: int,includeFilter: DataGridViewElementStates) -> int

  

   Returns the index of the previous System.Windows.Forms.DataGridViewRow that meets the specified 

    criteria.

  

  

   indexStart: The index of the row where the method should begin to look for the previous 

    System.Windows.Forms.DataGridViewRow.

  

   includeFilter: A bitwise combination of System.Windows.Forms.DataGridViewElementStates values.

   Returns: The index of the previous System.Windows.Forms.DataGridViewRow that has the attributes specified 

    by includeFilter; -1 if no row is found.
  """
  pass
 def GetRowCount(self,includeFilter):
  """
  GetRowCount(self: DataGridViewRowCollection,includeFilter: DataGridViewElementStates) -> int

  

   Returns the number of System.Windows.Forms.DataGridViewRow objects in the collection that meet 

    the specified criteria.

  

  

   includeFilter: A bitwise combination of System.Windows.Forms.DataGridViewElementStates values.

   Returns: The number of System.Windows.Forms.DataGridViewRow objects in the 

    System.Windows.Forms.DataGridViewRowCollection that have the attributes specified by 

    includeFilter.
  """
  pass
 def GetRowsHeight(self,includeFilter):
  """
  GetRowsHeight(self: DataGridViewRowCollection,includeFilter: DataGridViewElementStates) -> int

  

   Returns the cumulative height of the System.Windows.Forms.DataGridViewRow objects that meet the 

    specified criteria.

  

  

   includeFilter: A bitwise combination of System.Windows.Forms.DataGridViewElementStates values.

   Returns: The cumulative height of System.Windows.Forms.DataGridViewRow objects in the 

    System.Windows.Forms.DataGridViewRowCollection that have the attributes specified by 

    includeFilter.
  """
  pass
 def GetRowState(self,rowIndex):
  """
  GetRowState(self: DataGridViewRowCollection,rowIndex: int) -> DataGridViewElementStates

  

   Gets the state of the row with the specified index.

  

   rowIndex: The index of the row.

   Returns: A bitwise combination of System.Windows.Forms.DataGridViewElementStates values indicating the 

    state of the specified row.
  """
  pass
 def IndexOf(self,dataGridViewRow):
  """
  IndexOf(self: DataGridViewRowCollection,dataGridViewRow: DataGridViewRow) -> int

  

   Returns the index of a specified item in the collection.

  

   dataGridViewRow: The System.Windows.Forms.DataGridViewRow to locate in the 

    System.Windows.Forms.DataGridViewRowCollection.

  

   Returns: The index of value if it is a System.Windows.Forms.DataGridViewRow found in the 

    System.Windows.Forms.DataGridViewRowCollection; otherwise,-1.
  """
  pass
 def Insert(self,rowIndex,*__args):
  """
  Insert(self: DataGridViewRowCollection,rowIndex: int,count: int)

   Inserts the specified number of rows into the collection at the specified location.

  

   rowIndex: The position at which to insert the rows.

   count: The number of rows to insert into the System.Windows.Forms.DataGridViewRowCollection.

  Insert(self: DataGridViewRowCollection,rowIndex: int,dataGridViewRow: DataGridViewRow)

   Inserts the specified System.Windows.Forms.DataGridViewRow into the collection.

  

   rowIndex: The position at which to insert the row.

   dataGridViewRow: The System.Windows.Forms.DataGridViewRow to insert into the 

    System.Windows.Forms.DataGridViewRowCollection.

  

  Insert(self: DataGridViewRowCollection,rowIndex: int,*values: Array[object])

   Inserts a row into the collection at the specified position,and populates the cells with the 

    specified objects.

  

  

   rowIndex: The position at which to insert the row.

   values: A variable number of objects that populate the cells of the new row.
  """
  pass
 def InsertCopies(self,indexSource,indexDestination,count):
  """
  InsertCopies(self: DataGridViewRowCollection,indexSource: int,indexDestination: int,count: int)

   Inserts rows into the collection at the specified position.

  

   indexSource: The index of the System.Windows.Forms.DataGridViewRow on which to base the new rows.

   indexDestination: The position at which to insert the rows.

   count: The number of System.Windows.Forms.DataGridViewRow objects to add to the 

    System.Windows.Forms.DataGridViewRowCollection.
  """
  pass
 def InsertCopy(self,indexSource,indexDestination):
  """
  InsertCopy(self: DataGridViewRowCollection,indexSource: int,indexDestination: int)

   Inserts a row into the collection at the specified position,based on the row at specified 

    position.

  

  

   indexSource: The index of the row on which to base the new row.

   indexDestination: The position at which to insert the row.
  """
  pass
 def InsertRange(self,rowIndex,dataGridViewRows):
  """
  InsertRange(self: DataGridViewRowCollection,rowIndex: int,*dataGridViewRows: Array[DataGridViewRow])

   Inserts the System.Windows.Forms.DataGridViewRow objects into the collection at the specified 

    position.

  

  

   rowIndex: The position at which to insert the rows.

   dataGridViewRows: An array of System.Windows.Forms.DataGridViewRow objects to add to the 

    System.Windows.Forms.DataGridViewRowCollection.
  """
  pass
 def OnCollectionChanged(self,*args):
  """
  OnCollectionChanged(self: DataGridViewRowCollection,e: CollectionChangeEventArgs)

   Raises the System.Windows.Forms.DataGridViewRowCollection.CollectionChanged event.

  

   e: A System.ComponentModel.CollectionChangeEventArgs that contains the event data.
  """
  pass
 def Remove(self,dataGridViewRow):
  """
  Remove(self: DataGridViewRowCollection,dataGridViewRow: DataGridViewRow)

   Removes the row from the collection.

  

   dataGridViewRow: The row to remove from the System.Windows.Forms.DataGridViewRowCollection.
  """
  pass
 def RemoveAt(self,index):
  """
  RemoveAt(self: DataGridViewRowCollection,index: int)

   Removes the row at the specified position from the collection.

  

   index: The position of the row to remove.
  """
  pass
 def SharedRow(self,rowIndex):
  """
  SharedRow(self: DataGridViewRowCollection,rowIndex: int) -> DataGridViewRow

  

   Returns the System.Windows.Forms.DataGridViewRow at the specified index.

  

   rowIndex: The index of the System.Windows.Forms.DataGridViewRow to get.

   Returns: The System.Windows.Forms.DataGridViewRow positioned at the specified index.
  """
  pass
 def __add__(self,*args):
  """ x.__add__(y) <==> x+yx.__add__(y) <==> x+yx.__add__(y) <==> x+yx.__add__(y) <==> x+y """
  pass
 def __contains__(self,*args):
  """
  __contains__(self: IList,value: object) -> bool

  

   Determines whether the System.Collections.IList contains a specific value.

  

   value: The object to locate in the System.Collections.IList.

   Returns: true if the System.Object is found in the System.Collections.IList; otherwise,false.
  """
  pass
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y] """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __iter__(self,*args):
  """ __iter__(self: IEnumerable) -> object """
  pass
 def __len__(self,*args):
  """ x.__len__() <==> len(x) """
  pass
 @staticmethod
 def __new__(self,dataGridView):
  """ __new__(cls: type,dataGridView: DataGridView) """
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 Count=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of rows in the collection.



Get: Count(self: DataGridViewRowCollection) -> int



"""

 DataGridView=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Forms.DataGridView that owns the collection.



"""

 List=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets an array of System.Windows.Forms.DataGridViewRow objects.



"""


 CollectionChanged=None




class ControlTemplate(FrameworkTemplate,INameScope,ISealable,IHaveResources,IQueryAmbient):
 """
 Specifies the visual structure and behavioral aspects of a System.Windows.Controls.Control that can be shared across multiple instances of the control.

 

 ControlTemplate()

 ControlTemplate(targetType: Type)
 """
 def ValidateTemplatedParent(self,*args):
  """
  ValidateTemplatedParent(self: ControlTemplate,templatedParent: FrameworkElement)

   Checks the templated parent against a set of rules.

  

   templatedParent: The element this template is applied to.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,targetType=None):
  """
  __new__(cls: type)

  __new__(cls: type,targetType: Type)
  """
  pass
 TargetType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the type for which this System.Windows.Controls.ControlTemplate is intended.



Get: TargetType(self: ControlTemplate) -> Type



Set: TargetType(self: ControlTemplate)=value

"""

 Triggers=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a collection of System.Windows.TriggerBase objects that apply property changes or perform actions based on specified conditions.



Get: Triggers(self: ControlTemplate) -> TriggerCollection



"""





# -*- coding: UTF-8 -*-

""" Counting functions for Revit elements. """

from pyrevit import DB
from pyrevit.compat import get_elementid_value_func
import pyrevit.revit.db.query as q


def count_unpinned_revit_links(revitlinks_elements):
    """
    Returns the number of unpinned Revit links in the document.

    Args:
        rvtlinks_elements (list): A list of Revit link elements.

    Returns:
        int: The number of unpinned Revit links in the document.
    """
    return sum(
        1
        for rvt_link in revitlinks_elements
        if hasattr(rvt_link, "Pinned") and not rvt_link.Pinned
    )


def count_unplaced_rooms(rooms):
    """
    Counts the number of unplaced rooms in a given list of rooms.
    An unplaced room is defined as a room with no location (i.e., room.Location is None).

    Args:
        rooms (list, optional): A list of room objects. Defaults to None.

    Returns:
        int: The number of unplaced rooms in the list.
    """
    return sum(1 for room in rooms if room.Location is None)


def count_unbounded_rooms(rooms):
    """
    Counts the number of unbounded rooms (rooms with an area of 0) in the given list of rooms.

    Args:
        rooms (list, optional): A list of room objects. Each room object should have an 'Area' attribute.

    Returns:
        int: The number of unbounded rooms in the list.
    """
    return sum(1 for room in rooms if room.Area == 0)


def count_analytical_model_activated(document):
    """
    Returns the number of activated analytical models in the document.

    Args:
    document (Document): A Revit document.

    Returns:
    int: The number of elements with the analytical model activated in the document.
    """
    param = DB.BuiltInParameter.STRUCTURAL_ANALYTICAL_MODEL
    provider = DB.ParameterValueProvider(DB.ElementId(param))
    evaluator = DB.FilterNumericEquals()
    rule = DB.FilterIntegerRule(provider, evaluator, 1)
    elem_param_filter = DB.ElementParameterFilter(rule)
    return (
        DB.FilteredElementCollector(document)
        .WherePasses(elem_param_filter)
        .GetElementCount()
    )


def count_copied_views(views_set):
    """
    Returns the number of views in the given set that have "Copy" or "Copie" in their name.

    Args:
    views_set (set): A set of views to check for copied views. Defaults to None.

    Returns:
    int: The number of views in the set that have "Copy" or "Copie" in their name.
    """
    copied_view_names = ["Copy", "Copie"]
    copied_views_count = 0
    for view in views_set:
        view_name = q.get_name(view)
        try:
            # FIXME French compatibility, make it universal
            if any(name in view_name for name in copied_view_names):
                copied_views_count += 1
        except Exception as e:
            print(e)
    return copied_views_count


def count_unused_view_templates(views_list, document):
    """
    Returns the count of unused view templates in the given list of views.

    Args:
        views (list): A list of views to check for unused view templates.

    Returns:
        int: The count of unused view templates.
    """
    if views_list is None:
        return 0
    applied_templates = [v.ViewTemplateId for v in views_list if hasattr(v, "ViewTemplateId")]
    view_templates_list = q.get_all_view_templates(doc=document)
    unused_view_templates = []
    for v in view_templates_list:
        if v.Id not in applied_templates:
            unused_view_templates.append(v)
    return len(unused_view_templates)


def count_unused_filters_in_views(view_list, filters):
    """
    Counts the number of unused filters in the given list of views.

    Args:
        view_list (list): A list of Revit view objects to check for filter usage.
        filters (list): A list of filter objects to check against the views.

        Returns:
        int: The number of filters that are not used in any of the provided views.
    """
    used_filters_set = set()
    all_filters = set()
    get_elementid_value = get_elementid_value_func()
    for flt in filters:
        all_filters.add(get_elementid_value(flt.Id))
    for v in view_list:
        if hasattr(v, "AreGraphicsOverridesAllowed") and v.AreGraphicsOverridesAllowed():
            view_filters = v.GetFilters()
            for filter_id in view_filters:
                used_filters_set.add(get_elementid_value(filter_id))
    unused_view_filters_count = len(all_filters - used_filters_set)
    return unused_view_filters_count


def count_linked_dwg_files(document):
    """
    Returns the number of linked DWG files in the document.

    Args:
        document (DB.Document): The Revit document to search for linked DWG files.

    Returns:
        int: The number of linked DWG files in the document.
    """
    dwg_collector = q.get_elements_by_class(DB.ImportInstance, doc=document)
    if not dwg_collector:
        return 0
    dwg_imported = 0
    for dwg in dwg_collector:
        if not dwg.IsLinked:
            dwg_imported += 1
    return len(dwg_collector) - dwg_imported


def count_in_place_families(document):
    """
    This function collects all the in-place families in the given document and returns their count.

    Args:
    - document: The Revit document to search for in-place families in.

    Returns:
    - int: The count of in-place families.
    """
    familyinstance_collector = q.get_elements_by_class(DB.FamilyInstance, doc=document)
    in_place_family_count = sum(
        1
        for x in familyinstance_collector
        if x.Symbol and x.Symbol.Family and x.Symbol.Family.IsInPlace
    )
    return in_place_family_count


def count_total_families(document):
    """
    Counts the total number of unique family names in the given Revit document.

    Args:
        document: The Revit document to count families from.

    Returns:
        int: The total number of unique family names in the document.
    """
    families_collection = q.get_families(document, only_editable=True)
    unique_families = []
    for fam in families_collection:
        if fam.Name not in unique_families:
            unique_families.append(fam.Name)
    if not unique_families:
        return 0
    return len(unique_families)


def count_textnote_types_with_changed_width_factor(text_notes_types):
    """
    Counts the number of text note types that have a width factor different from the default value of 1.

    Args:
        text_notes_types (list): A list of text note types to check.

    Returns:
        int: The number of text note types with a changed width factor.
    """
    if text_notes_types is None:
        return 0
    changed_width_factor = 0
    for textnote in text_notes_types:
        width_factor = textnote.get_Parameter(
            DB.BuiltInParameter.TEXT_WIDTH_SCALE
        ).AsDouble()
        if width_factor != 1:
            changed_width_factor += 1
    return changed_width_factor


def count_textnote_types_with_opaque_background(text_notes_types):
    """
    Counts the number of text notes with an opaque background.

    Args:
        text_notes_types (list): A list of text note elements.

    Returns:
        int: The number of text notes with an opaque background.
    """
    if text_notes_types is None:
        return 0
    text_opaque_background = 0
    for textnote in text_notes_types:
        text_background = textnote.get_Parameter(
            DB.BuiltInParameter.TEXT_BACKGROUND
        ).AsInteger()
        if text_background == 0:
            text_opaque_background += 1
    return text_opaque_background


def count_text_notes_with_all_caps(text_notes):
    """
    Counts the number of text notes that have all capital letters.

    Args:
        text_notes (list): A list of text note objects.

    Returns:
        int: The number of text notes with all capital letters.
    """
    if text_notes is None:
        return 0
    caps_count = 0
    for text_note in text_notes:
        caps_status = text_note.GetFormattedText().GetAllCapsStatus()
        if caps_status:
            caps_count += 1
    return caps_count


def count_unnamed_reference_planes(reference_planes):
    """
    Returns the count of unnamed reference planes in the given document.

    Args:
        document (DB.Document): The document to search for reference planes. Defaults to the active document.

    Returns:
        int: The count of unnamed reference planes.
    """
    if reference_planes is None:
        return 0
    unnamed_ref_planes_count = 0
    # Default reference plane label, not the most elegant solution
    reference_plane_default_label = DB.LabelUtils.GetLabelFor(
        DB.BuiltInCategory.OST_CLines
    ).replace("s", "")
    for ref_plane in reference_planes:
        if ref_plane.Name == reference_plane_default_label:
            unnamed_ref_planes_count += 1
    return unnamed_ref_planes_count


def count_dimension_overrides(dimensions):
    """
    Counts the number of dimension overrides in a list of dimensions.
    This function iterates through a list of dimensions and counts how many of them
    have a value override. It also checks for overrides in dimension segments if they exist.

    Args:
        dimensions (list): A list of dimension objects to check for overrides.

    Returns:
        int: The total count of dimension overrides found in the list of dimensions.
    """
    if dimensions is None:
        return 0
    dim_overrides_count = 0
    for d in dimensions:
        if d.ValueOverride is not None:
            dim_overrides_count += 1
        if d.Segments:
            for seg in d.Segments:
                if seg.ValueOverride:
                    dim_overrides_count += 1
    return dim_overrides_count


def count_revision_clouds(document):
    """
    Returns the number of revision clouds in the given document.

    Args:
        document (DB.Document): The document to search for revision clouds in. Defaults to the active document.

    Returns:
        int: The number of revision clouds in the document.
    """
    return (
        DB.FilteredElementCollector(document)
        .OfCategory(DB.BuiltInCategory.OST_RevisionClouds)
        .WhereElementIsNotElementType()
        .GetElementCount()
    )


def count_groups(doc):
    """
    Counts the instances and types of model and detail groups in a Revit document, excluding array members.
    
    Args:
        doc (DB.Document): The Revit document to process.
    
    Returns:
        tuple: A tuple containing four integers:
            - model_group_instances_count (int): The count of model group instances.
            - model_group_types_count (int): The count of model group types.
            - detail_group_instances_count (int): The count of detail group instances.
            - detail_group_types_count (int): The count of detail group types.
    """
    arrays_members_collector = DB.FilteredElementCollector(doc).OfCategory(DB.BuiltInCategory.OST_IOSArrays).ToElements()
    arraysmembers = set()
    for array in arrays_members_collector:
        if array.OwnerViewId != DB.ElementId.InvalidElementId:
            arraysmembers.update(array.GetOriginalMemberIds())
            arraysmembers.update(array.GetCopiedMemberIds())

    arrays_grouptype_members = set(doc.GetElement(array).GetTypeId() for array in arraysmembers)


    model_groups = DB.FilteredElementCollector(doc).OfCategory(DB.BuiltInCategory.OST_IOSModelGroups).ToElements()
    model_group_instances_count = 0
    model_group_types_count = 0
    for model_group in model_groups:
        if type(model_group) is DB.GroupType and model_group.Id not in arrays_grouptype_members:
            model_group_types_count += 1
        elif type(model_group) is DB.Group and model_group.Id not in arraysmembers:
            model_group_instances_count += 1

    detail_groups = DB.FilteredElementCollector(doc).OfCategory(DB.BuiltInCategory.OST_IOSDetailGroups).ToElements()
    detail_group_instances_count = 0
    detail_group_types_count = 0
    for detail_group in detail_groups:
        if type(detail_group) is DB.GroupType and detail_group.Id not in arrays_grouptype_members:
            detail_group_types_count += 1
        elif type(detail_group) is DB.Group and detail_group.Id not in arraysmembers:
            detail_group_instances_count += 1
    return model_group_instances_count, model_group_types_count, detail_group_instances_count, detail_group_types_count



class FormBorderStyle(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the border styles for a form.

 

 enum FormBorderStyle,values: Fixed3D (2),FixedDialog (3),FixedSingle (1),FixedToolWindow (5),None (0),Sizable (4),SizableToolWindow (6)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Fixed3D=None
 FixedDialog=None
 FixedSingle=None
 FixedToolWindow=None
 None=None
 Sizable=None
 SizableToolWindow=None
 value__=None




class AccessibleObject(StandardOleMarshalObject,IMarshal,IReflect,IAccessible,IEnumVariant,IOleWindow):
 """
 Provides information that accessibility applications use to adjust an application's user interface (UI) for users with impairments.

 

 AccessibleObject()
 """
 def DoDefaultAction(self):
  """
  DoDefaultAction(self: AccessibleObject)

   Performs the default action associated with this accessible object.
  """
  pass
 def GetChild(self,index):
  """
  GetChild(self: AccessibleObject,index: int) -> AccessibleObject

  

   Retrieves the accessible child corresponding to the specified index.

  

   index: The zero-based index of the accessible child.

   Returns: An System.Windows.Forms.AccessibleObject that represents the accessible child corresponding to 

    the specified index.
  """
  pass
 def GetChildCount(self):
  """
  GetChildCount(self: AccessibleObject) -> int

  

   Retrieves the number of children belonging to an accessible object.

   Returns: The number of children belonging to an accessible object.
  """
  pass
 def GetFocused(self):
  """
  GetFocused(self: AccessibleObject) -> AccessibleObject

  

   Retrieves the object that has the keyboard focus.

   Returns: An System.Windows.Forms.AccessibleObject that specifies the currently focused child. This method 

    returns the calling object if the object itself is focused. Returns null if no object has focus.
  """
  pass
 def GetHelpTopic(self,fileName):
  """
  GetHelpTopic(self: AccessibleObject) -> (int,str)

  

   Gets an identifier for a Help topic identifier and the path to the Help file associated with 

    this accessible object.

  

   Returns: An identifier for a Help topic,or -1 if there is no Help topic. On return,the fileName 

    parameter contains the path to the Help file associated with this accessible object.
  """
  pass
 def GetSelected(self):
  """
  GetSelected(self: AccessibleObject) -> AccessibleObject

  

   Retrieves the currently selected child.

   Returns: An System.Windows.Forms.AccessibleObject that represents the currently selected child. This 

    method returns the calling object if the object itself is selected. Returns null if is no child 

    is currently selected and the object itself does not have focus.
  """
  pass
 def HitTest(self,x,y):
  """
  HitTest(self: AccessibleObject,x: int,y: int) -> AccessibleObject

  

   Retrieves the child object at the specified screen coordinates.

  

   x: The horizontal screen coordinate.

   y: The vertical screen coordinate.

   Returns: An System.Windows.Forms.AccessibleObject that represents the child object at the given screen 

    coordinates. This method returns the calling object if the object itself is at the location 

    specified. Returns null if no object is at the tested location.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def Navigate(self,navdir):
  """
  Navigate(self: AccessibleObject,navdir: AccessibleNavigation) -> AccessibleObject

  

   Navigates to another accessible object.

  

   navdir: One of the System.Windows.Forms.AccessibleNavigation values.

   Returns: An System.Windows.Forms.AccessibleObject that represents one of the 

    System.Windows.Forms.AccessibleNavigation values.
  """
  pass
 def Select(self,flags):
  """
  Select(self: AccessibleObject,flags: AccessibleSelection)

   Modifies the selection or moves the keyboard focus of the accessible object.

  

   flags: One of the System.Windows.Forms.AccessibleSelection values.
  """
  pass
 def UseStdAccessibleObjects(self,*args):
  """
  UseStdAccessibleObjects(self: AccessibleObject,handle: IntPtr,objid: int)

   Associates an object with an instance of an System.Windows.Forms.AccessibleObject based on the 

    handle and the object id of the object.

  

  

   handle: An System.IntPtr that contains the handle of the object.

   objid: An Int that defines the type of object that the handle parameter refers to.

  UseStdAccessibleObjects(self: AccessibleObject,handle: IntPtr)

   Associates an object with an instance of an System.Windows.Forms.AccessibleObject based on the 

    handle of the object.

  

  

   handle: An System.IntPtr that contains the handle of the object.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 Bounds=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the location and size of the accessible object.



Get: Bounds(self: AccessibleObject) -> Rectangle



"""

 DefaultAction=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a string that describes the default action of the object. Not all objects have a default action.



Get: DefaultAction(self: AccessibleObject) -> str



"""

 Description=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a string that describes the visual appearance of the specified object. Not all objects have a description.



Get: Description(self: AccessibleObject) -> str



"""

 Help=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a description of what the object does or how the object is used.



Get: Help(self: AccessibleObject) -> str



"""

 KeyboardShortcut=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the shortcut key or access key for the accessible object.



Get: KeyboardShortcut(self: AccessibleObject) -> str



"""

 Name=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the object name.



Get: Name(self: AccessibleObject) -> str



Set: Name(self: AccessibleObject)=value

"""

 Parent=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the parent of an accessible object.



Get: Parent(self: AccessibleObject) -> AccessibleObject



"""

 Role=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the role of this accessible object.



Get: Role(self: AccessibleObject) -> AccessibleRole



"""

 State=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the state of this accessible object.



Get: State(self: AccessibleObject) -> AccessibleStates



"""

 Value=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the value of an accessible object.



Get: Value(self: AccessibleObject) -> str



Set: Value(self: AccessibleObject)=value

"""





class CalendarDateChangedEventArgs(RoutedEventArgs):
 """ Provides data for the System.Windows.Controls.Calendar.DisplayDateChanged event. """
 AddedDate=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the date to be newly displayed.



Get: AddedDate(self: CalendarDateChangedEventArgs) -> Nullable[DateTime]



"""

 RemovedDate=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the date that was previously displayed.



Get: RemovedDate(self: CalendarDateChangedEventArgs) -> Nullable[DateTime]



"""





class UploadStringCompletedEventArgs(AsyncCompletedEventArgs):
 """ Provides data for the System.Net.WebClient.UploadStringCompleted event. """
 Result=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the server reply to a string upload operation that is started by calling an erload:System.Net.WebClient.UploadStringAsync method.



Get: Result(self: UploadStringCompletedEventArgs) -> str



"""





class ComponentEditor(object):
 """ Provides the base class for a custom component editor. """
 def EditComponent(self,*__args):
  """
  EditComponent(self: ComponentEditor,context: ITypeDescriptorContext,component: object) -> bool

  

   Edits the component and returns a value indicating whether the component was modified based upon 

    a given context.

  

  

   context: An optional context object that can be used to obtain further information about the edit.

   component: The component to be edited.

   Returns: true if the component was modified; otherwise,false.

  EditComponent(self: ComponentEditor,component: object) -> bool

  

   Edits the component and returns a value indicating whether the component was modified.

  

   component: The component to be edited.

   Returns: true if the component was modified; otherwise,false.
  """
  pass



class PrimaryInteropAssemblyAttribute(Attribute,_Attribute):
 """
 Indicates that the attributed assembly is a primary interop assembly.

 

 PrimaryInteropAssemblyAttribute(major: int,minor: int)
 """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,major,minor):
  """ __new__(cls: type,major: int,minor: int) """
  pass
 MajorVersion=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the major version number of the type library for which this assembly is the primary interop assembly.



Get: MajorVersion(self: PrimaryInteropAssemblyAttribute) -> int



"""

 MinorVersion=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the minor version number of the type library for which this assembly is the primary interop assembly.



Get: MinorVersion(self: PrimaryInteropAssemblyAttribute) -> int



"""





# encoding: utf-8
# module System.Windows.Media.TextFormatting calls itself TextFormatting
# from PresentationCore,Version=4.0.0.0,Culture=neutral,PublicKeyToken=31bf3856ad364e35
# by generator 1.145
""" NamespaceTracker represent a CLS namespace. """
# no imports

# no functions
# classes

class CharacterBufferRange(object,IEquatable[CharacterBufferRange]):
 """
 Describes a string of characters.

 

 CharacterBufferRange(characterArray: Array[Char],offsetToFirstChar: int,characterLength: int)

 CharacterBufferRange(characterString: str,offsetToFirstChar: int,characterLength: int)

 CharacterBufferRange(unsafeCharacterString: Char*,characterLength: int)
 """
 def Equals(self,*__args):
  """
  Equals(self: CharacterBufferRange,value: CharacterBufferRange) -> bool

  

   Determines whether the System.Windows.Media.TextFormatting.CharacterBufferRange object is equal 

    to the current System.Windows.Media.TextFormatting.CharacterBufferRange object.

  

  

   value: The System.Windows.Media.TextFormatting.CharacterBufferRange to compare with the current 

    System.Windows.Media.TextFormatting.CharacterBufferRange object.

  

   Returns: true if value is equal to the current System.Windows.Media.TextFormatting.CharacterBufferRange 

    object; otherwise,false. If value is not a 

    System.Windows.Media.TextFormatting.CharacterBufferRange object,false is returned.

  

  Equals(self: CharacterBufferRange,obj: object) -> bool

  

   Determines whether the specified object is equal to the current 

    System.Windows.Media.TextFormatting.CharacterBufferRange object.

  

  

   obj: The System.Object to compare with the current 

    System.Windows.Media.TextFormatting.CharacterBufferRange object.

  

   Returns: true if o is equal to the current System.Windows.Media.TextFormatting.CharacterBufferRange 

    object; otherwise,false. If o is not a System.Windows.Media.TextFormatting.CharacterBufferRange 

    object,false is returned.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: CharacterBufferRange) -> int

  

   Serves as a hash function for System.Windows.Media.TextFormatting.CharacterBufferRange. It is 

    suitable for use in hashing algorithms and data structures such as a hash table.

  

   Returns: An System.Int32 value that represents the hash code for the current object.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type,characterArray: Array[Char],offsetToFirstChar: int,characterLength: int)

  __new__(cls: type,characterString: str,offsetToFirstChar: int,characterLength: int)

  __new__(cls: type,unsafeCharacterString: Char*,characterLength: int)

  __new__[CharacterBufferRange]() -> CharacterBufferRange
  """
  pass
 def __ne__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 def __str__(self,*args):
  pass
 CharacterBufferReference=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a reference to the character buffer of a string.



Get: CharacterBufferReference(self: CharacterBufferRange) -> CharacterBufferReference



"""

 Length=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of characters in the text source character store.



Get: Length(self: CharacterBufferRange) -> int



"""


 Empty=None


class CharacterBufferReference(object,IEquatable[CharacterBufferReference]):
 """
 Describes a character buffer for a text run.

 

 CharacterBufferReference(characterString: str,offsetToFirstChar: int)

 CharacterBufferReference(characterArray: Array[Char],offsetToFirstChar: int)

 CharacterBufferReference(unsafeCharacterString: Char*,characterLength: int)
 """
 def Equals(self,*__args):
  """
  Equals(self: CharacterBufferReference,value: CharacterBufferReference) -> bool

  

   Determines whether the System.Windows.Media.TextFormatting.CharacterBufferReference is equal to 

    the current System.Windows.Media.TextFormatting.CharacterBufferReference object.

  

  

   value: The System.Windows.Media.TextFormatting.CharacterBufferReference to compare with the current 

    System.Windows.Media.TextFormatting.CharacterBufferReference object.

  

   Returns: true if value is equal to the current 

    System.Windows.Media.TextFormatting.CharacterBufferReference object; otherwise,false.

  

  Equals(self: CharacterBufferReference,obj: object) -> bool

  

   Determines whether the specified object is equal to the current 

    System.Windows.Media.TextFormatting.CharacterBufferReference object.

  

  

   obj: The object to compare with the current 

    System.Windows.Media.TextFormatting.CharacterBufferReference object.

  

   Returns: true if obj is equal to the current System.Windows.Media.TextFormatting.CharacterBufferReference 

    object; otherwise,false. If obj is not a 

    System.Windows.Media.TextFormatting.CharacterBufferReference object,false is returned.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: CharacterBufferReference) -> int

  

   Serves as a hash function for System.Windows.Media.TextFormatting.CharacterBufferReference. It 

    is suitable for use in hashing algorithms and data structures such as a hash table.

  

   Returns: An System.Int32 value that represents the hash code for the current object.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type,characterArray: Array[Char],offsetToFirstChar: int)

  __new__(cls: type,characterString: str,offsetToFirstChar: int)

  __new__(cls: type,unsafeCharacterString: Char*,characterLength: int)

  __new__[CharacterBufferReference]() -> CharacterBufferReference
  """
  pass
 def __ne__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 def __str__(self,*args):
  pass

class CharacterHit(object,IEquatable[CharacterHit]):
 """
 Represents information about a character hit within a glyph run.

 

 CharacterHit(firstCharacterIndex: int,trailingLength: int)
 """
 def Equals(self,obj):
  """
  Equals(self: CharacterHit,obj: object) -> bool

  

   Determines whether the specified object is equal to the current 

    System.Windows.Media.TextFormatting.CharacterBufferReference object.

  

  

   obj: The object to compare with the current 

    System.Windows.Media.TextFormatting.CharacterBufferReference object.

  

   Returns: true if obj is equal to the current System.Windows.Media.TextFormatting.CharacterBufferReference 

    object; otherwise,false. If obj is not a 

    System.Windows.Media.TextFormatting.CharacterBufferReference object,false is returned.

  

  Equals(self: CharacterHit,obj: CharacterHit) -> bool

  

   Determines whether the System.Windows.Media.TextFormatting.CharacterBufferReference is equal to 

    the current System.Windows.Media.TextFormatting.CharacterBufferReference object.

  

  

   obj: The System.Windows.Media.TextFormatting.CharacterBufferReference to compare with the current 

    System.Windows.Media.TextFormatting.CharacterBufferReference object.

  

   Returns: true if obj is equal to the current System.Windows.Media.TextFormatting.CharacterBufferReference 

    object; otherwise,false.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: CharacterHit) -> int

  

   Serves as a hash function for System.Windows.Media.TextFormatting.CharacterBufferReference. It 

    is suitable for use in hashing algorithms and data structures such as a hash table.

  

   Returns: An System.Int32 value that represents the hash code for the current object.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,firstCharacterIndex,trailingLength):
  """
  __new__(cls: type,firstCharacterIndex: int,trailingLength: int)

  __new__[CharacterHit]() -> CharacterHit
  """
  pass
 def __ne__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 def __str__(self,*args):
  pass
 FirstCharacterIndex=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the index of the first character that got hit.



Get: FirstCharacterIndex(self: CharacterHit) -> int



"""

 TrailingLength=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the trailing length value for the character that got hit.



Get: TrailingLength(self: CharacterHit) -> int



"""



class CultureSpecificCharacterBufferRange(object):
 """
 Represents a range of characters that are associated with a culture.

 

 CultureSpecificCharacterBufferRange(culture: CultureInfo,characterBufferRange: CharacterBufferRange)
 """
 @staticmethod
 def __new__(self,culture,characterBufferRange):
  """ __new__(cls: type,culture: CultureInfo,characterBufferRange: CharacterBufferRange) """
  pass
 CharacterBufferRange=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Media.TextFormatting.CharacterBufferRange of the System.Windows.Media.TextFormatting.CultureSpecificCharacterBufferRange.



Get: CharacterBufferRange(self: CultureSpecificCharacterBufferRange) -> CharacterBufferRange



"""

 CultureInfo=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Globalization.CultureInfo of the System.Windows.Media.TextFormatting.CultureSpecificCharacterBufferRange.



Get: CultureInfo(self: CultureSpecificCharacterBufferRange) -> CultureInfo



"""



class IndexedGlyphRun(object):
 """ Allows text engine clients to map a text source character index to the corresponding System.Windows.Media.GlyphRun. """
 GlyphRun=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Media.GlyphRun that corresponds to the System.Windows.Media.TextFormatting.IndexedGlyphRun object.



Get: GlyphRun(self: IndexedGlyphRun) -> GlyphRun



"""

 TextSourceCharacterIndex=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the text source character index that corresponds to the beginning of the System.Windows.Media.GlyphRun.



Get: TextSourceCharacterIndex(self: IndexedGlyphRun) -> int



"""

 TextSourceLength=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the text source character length that corresponds to the System.Windows.Media.TextFormatting.IndexedGlyphRun object.



Get: TextSourceLength(self: IndexedGlyphRun) -> int



"""



class InvertAxes(Enum,IComparable,IFormattable,IConvertible):
 """
 Indicate the inversion of horizontal and vertical axes of the drawing surface.

 

 enum (flags) InvertAxes,values: Both (3),Horizontal (1),None (0),Vertical (2)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Both=None
 Horizontal=None
 None=None
 value__=None
 Vertical=None


class MinMaxParagraphWidth(object,IEquatable[MinMaxParagraphWidth]):
 """ Represents the smallest and largest possible paragraph width that can fully contain the specified text content. """
 def Equals(self,*__args):
  """
  Equals(self: MinMaxParagraphWidth,obj: object) -> bool

  

   Determines whether the specified object is equal to the current 

    System.Windows.Media.TextFormatting.CharacterBufferReference object.

  

  

   obj: The System.Object to compare with the current 

    System.Windows.Media.TextFormatting.CharacterBufferReference object.

  

   Returns: true if obj is equal to the current System.Windows.Media.TextFormatting.CharacterBufferReference 

    object; otherwise,false. If obj is not a 

    System.Windows.Media.TextFormatting.CharacterBufferReference object,false is returned.

  

  Equals(self: MinMaxParagraphWidth,value: MinMaxParagraphWidth) -> bool

  

   Determines whether the System.Windows.Media.TextFormatting.CharacterBufferReference is equal to 

    the current System.Windows.Media.TextFormatting.CharacterBufferReference object.

  

  

   value: The System.Windows.Media.TextFormatting.CharacterBufferReference to compare with the current 

    System.Windows.Media.TextFormatting.CharacterBufferReference object.

  

   Returns: true if value is equal to the current 

    System.Windows.Media.TextFormatting.CharacterBufferReference object; otherwise,false.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: MinMaxParagraphWidth) -> int

  

   Serves as a hash function for System.Windows.Media.TextFormatting.CharacterBufferReference. It 

    is suitable for use in hashing algorithms and data structures such as a hash table.

  

   Returns: An System.Int32 value that represents the hash code for the current object.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __ne__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 def __str__(self,*args):
  pass
 MaxWidth=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the largest paragraph width possible that can fully contain the specified text content.



Get: MaxWidth(self: MinMaxParagraphWidth) -> float



"""

 MinWidth=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the smallest paragraph width possible that can fully contain the specified text content.



Get: MinWidth(self: MinMaxParagraphWidth) -> float



"""



class TextBounds(object):
 """ Represents the bounding rectangle of a range of characters. """
 FlowDirection=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the text flow direction for the System.Windows.Media.TextFormatting.TextBounds object.



Get: FlowDirection(self: TextBounds) -> FlowDirection



"""

 Rectangle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the bounding rectangle for the System.Windows.Media.TextFormatting.TextBounds object.



Get: Rectangle(self: TextBounds) -> Rect



"""

 TextRunBounds=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a list of System.Windows.Media.TextFormatting.TextRunBounds objects.



Get: TextRunBounds(self: TextBounds) -> IList[TextRunBounds]



"""



class TextRun(object):
 """ Represents a sequence of characters that share a single property set. """
 CharacterBufferReference=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a reference to the text run character buffer.



Get: CharacterBufferReference(self: TextRun) -> CharacterBufferReference



"""

 Length=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of characters in the text run.



Get: Length(self: TextRun) -> int



"""

 Properties=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the set of text properties that are shared by every character in the text run,such as typeface or foreground brush.



Get: Properties(self: TextRun) -> TextRunProperties



"""



class TextCharacters(TextRun,ITextSymbols,IShapeableTextCollector):
 """
 Represents a collection of character glyphs from distinct physical typefaces.

 

 TextCharacters(characterArray: Array[Char],offsetToFirstChar: int,length: int,textRunProperties: TextRunProperties)

 TextCharacters(characterString: str,textRunProperties: TextRunProperties)

 TextCharacters(characterString: str,offsetToFirstChar: int,length: int,textRunProperties: TextRunProperties)

 TextCharacters(unsafeCharacterString: Char*,length: int,textRunProperties: TextRunProperties)
 """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type,characterArray: Array[Char],offsetToFirstChar: int,length: int,textRunProperties: TextRunProperties)

  __new__(cls: type,characterString: str,textRunProperties: TextRunProperties)

  __new__(cls: type,characterString: str,offsetToFirstChar: int,length: int,textRunProperties: TextRunProperties)

  __new__(cls: type,unsafeCharacterString: Char*,length: int,textRunProperties: TextRunProperties)
  """
  pass
 CharacterBufferReference=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Media.TextFormatting.CharacterBufferReference for the System.Windows.Media.TextFormatting.TextCharacters.



Get: CharacterBufferReference(self: TextCharacters) -> CharacterBufferReference



"""

 Length=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the length of the text characters.



Get: Length(self: TextCharacters) -> int



"""

 Properties=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the set of properties shared by every text character.



Get: Properties(self: TextCharacters) -> TextRunProperties



"""



class TextCollapsedRange(object):
 """ Represents the range of characters and its width measurement for collapsed text within a line. """
 Length=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of characters for the collapsed text.



Get: Length(self: TextCollapsedRange) -> int



"""

 TextSourceCharacterIndex=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the index to the first character in the System.Windows.Media.TextFormatting.TextSource that represents collapsed text characters.



Get: TextSourceCharacterIndex(self: TextCollapsedRange) -> int



"""

 Width=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """The total width of collapsed text characters.



Get: Width(self: TextCollapsedRange) -> float



"""



class TextCollapsingProperties(object):
 """ Represents the characteristics of collapsed text. """
 Style=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the style of the collapsed text.



Get: Style(self: TextCollapsingProperties) -> TextCollapsingStyle



"""

 Symbol=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the text run that is used as the collapsed text symbol.



Get: Symbol(self: TextCollapsingProperties) -> TextRun



"""

 Width=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the width of the range of collapsed text.



Get: Width(self: TextCollapsingProperties) -> float



"""



class TextCollapsingStyle(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the style of collapsed text.

 

 enum TextCollapsingStyle,values: TrailingCharacter (0),TrailingWord (1)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 TrailingCharacter=None
 TrailingWord=None
 value__=None


class TextEmbeddedObject(TextRun):
 """ Defines a type of text content in which measuring,hit testing and drawing of the entire content is done in whole. """
 def ComputeBoundingBox(self,rightToLeft,sideways):
  """
  ComputeBoundingBox(self: TextEmbeddedObject,rightToLeft: bool,sideways: bool) -> Rect

  

   Gets the computed bounding box of the text object.

  

   rightToLeft: A System.Boolean value that determines if the text object is drawn from right to left.

   sideways: A System.Boolean value that determines if the text object is drawn with its side parallel to the 

    baseline.

  

   Returns: A System.Windows.Rect value that represents the bounding box size of the text object.
  """
  pass
 def Draw(self,drawingContext,origin,rightToLeft,sideways):
  """
  Draw(self: TextEmbeddedObject,drawingContext: DrawingContext,origin: Point,rightToLeft: bool,sideways: bool)

   Draws the text object.

  

   drawingContext: The System.Windows.Media.DrawingContext to use for rendering the text object.

   origin: The System.Windows.Point value that represents the origin where the text object is drawn.

   rightToLeft: A System.Boolean value that determines if the text object is drawn from right to left.

   sideways: A System.Boolean value that determines if the text object is drawn with its side parallel to the 

    baseline.
  """
  pass
 def Format(self,remainingParagraphWidth):
  """
  Format(self: TextEmbeddedObject,remainingParagraphWidth: float) -> TextEmbeddedObjectMetrics

  

   Get text object measurement metrics that will fit within the specified remaining width of the 

    paragraph.

  

  

   remainingParagraphWidth: A System.Double that represents the remaining paragraph width.

   Returns: A System.Windows.Media.TextFormatting.TextEmbeddedObjectMetrics value that represents the text 

    object metrics.
  """
  pass
 BreakAfter=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the line breaking condition after the text object.



Get: BreakAfter(self: TextEmbeddedObject) -> LineBreakCondition



"""

 BreakBefore=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the line breaking condition before the text object.



Get: BreakBefore(self: TextEmbeddedObject) -> LineBreakCondition



"""

 HasFixedSize=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Determines whether the text object has a fixed size regardless of where it is placed within a line.



Get: HasFixedSize(self: TextEmbeddedObject) -> bool



"""



class TextEmbeddedObjectMetrics(object):
 """
 Specifies properties for a System.Windows.Media.TextFormatting.TextEmbeddedObject.

 

 TextEmbeddedObjectMetrics(width: float,height: float,baseline: float)
 """
 @staticmethod
 def __new__(self,width,height,baseline):
  """ __new__(cls: type,width: float,height: float,baseline: float) """
  pass
 Baseline=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the baseline of the text object.



Get: Baseline(self: TextEmbeddedObjectMetrics) -> float



"""

 Height=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the height of the text object.



Get: Height(self: TextEmbeddedObjectMetrics) -> float



"""

 Width=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the width of the text object.



Get: Width(self: TextEmbeddedObjectMetrics) -> float



"""



class TextEndOfLine(TextRun):
 """
 Defines a specialized text run that is used to mark the end of a line.

 

 TextEndOfLine(length: int)

 TextEndOfLine(length: int,textRunProperties: TextRunProperties)
 """
 @staticmethod
 def __new__(self,length,textRunProperties=None):
  """
  __new__(cls: type,length: int)

  __new__(cls: type,length: int,textRunProperties: TextRunProperties)
  """
  pass
 CharacterBufferReference=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a reference to the System.Windows.Media.TextFormatting.TextEndOfLine character buffer.



Get: CharacterBufferReference(self: TextEndOfLine) -> CharacterBufferReference



"""

 Length=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the character length of the System.Windows.Media.TextFormatting.TextEndOfLine character buffer.



Get: Length(self: TextEndOfLine) -> int



"""

 Properties=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the set of properties shared by every text character of the System.Windows.Media.TextFormatting.TextEndOfLine character buffer.



Get: Properties(self: TextEndOfLine) -> TextRunProperties



"""



class TextEndOfParagraph(TextEndOfLine):
 """
 Defines a specialized text run that is used to mark the end of a paragraph.

 

 TextEndOfParagraph(length: int)

 TextEndOfParagraph(length: int,textRunProperties: TextRunProperties)
 """
 @staticmethod
 def __new__(self,length,textRunProperties=None):
  """
  __new__(cls: type,length: int)

  __new__(cls: type,length: int,textRunProperties: TextRunProperties)
  """
  pass

class TextEndOfSegment(TextRun):
 """
 Defines a specialized text run that is used to mark the end of a segment.

 

 TextEndOfSegment(length: int)
 """
 @staticmethod
 def __new__(self,length):
  """ __new__(cls: type,length: int) """
  pass
 CharacterBufferReference=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a reference to the System.Windows.Media.TextFormatting.TextEndOfSegment character buffer.



Get: CharacterBufferReference(self: TextEndOfSegment) -> CharacterBufferReference



"""

 Length=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the character length of the System.Windows.Media.TextFormatting.TextEndOfSegment character buffer.



Get: Length(self: TextEndOfSegment) -> int



"""

 Properties=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the set of properties shared by every text character of the System.Windows.Media.TextFormatting.TextEndOfSegment character buffer.



Get: Properties(self: TextEndOfSegment) -> TextRunProperties



"""



class TextFormatter(object,IDisposable):
 """ Provides services for formatting text and breaking text lines using a custom text layout client. """
 @staticmethod
 def Create(textFormattingMode=None):
  """
  Create() -> TextFormatter

  

   Creates a new instance of the System.Windows.Media.TextFormatting.TextFormatter class. This is a 

    static method.

  

   Returns: A new instance of System.Windows.Media.TextFormatting.TextFormatter.

  Create(textFormattingMode: TextFormattingMode) -> TextFormatter

  

   Creates a new instance of the System.Windows.Media.TextFormatting.TextFormatter class with the 

    specified formatting mode. This is a static method.

  

  

   textFormattingMode: The System.Windows.Media.TextFormattingMode that specifies the text layout for the 

    System.Windows.Media.TextFormatting.TextFormatter.

  

   Returns: A new instance of System.Windows.Media.TextFormatting.TextFormatter.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: TextFormatter)

   Releases all managed and unmanaged resources used by the 

    System.Windows.Media.TextFormatting.TextFormatter object.
  """
  pass
 def FormatLine(self,textSource,firstCharIndex,paragraphWidth,paragraphProperties,previousLineBreak,textRunCache=None):
  """
  FormatLine(self: TextFormatter,textSource: TextSource,firstCharIndex: int,paragraphWidth: float,paragraphProperties: TextParagraphProperties,previousLineBreak: TextLineBreak,textRunCache: TextRunCache) -> TextLine

  

   Creates a System.Windows.Media.TextFormatting.TextLine that is used for formatting and 

    displaying document content.

  

  

   textSource: A System.Windows.Media.TextFormatting.TextSource object that represents the text source for the 

    line.

  

   firstCharIndex: An System.Int32 value that specifies the character index of the starting character in the line.

   paragraphWidth: A System.Double value that specifies the width of the paragraph that the line fills.

   paragraphProperties: A System.Windows.Media.TextFormatting.TextParagraphProperties object that represents paragraph 

    properties,such as flow direction,alignment,or indentation.

  

   previousLineBreak: A System.Windows.Media.TextFormatting.TextLineBreak object that specifies the text formatter 

    state,in terms of where the previous line in the paragraph was broken by the text formatting 

    process.

  

   textRunCache: A System.Windows.Media.TextFormatting.TextRunCache object that represents the caching mechanism 

    for the layout of text.

  

   Returns: A System.Windows.Media.TextFormatting.TextLine value that represents a line of text that can be 

    displayed.

  

  FormatLine(self: TextFormatter,textSource: TextSource,firstCharIndex: int,paragraphWidth: float,paragraphProperties: TextParagraphProperties,previousLineBreak: TextLineBreak) -> TextLine

  

   Creates a System.Windows.Media.TextFormatting.TextLine that is used for formatting and 

    displaying document content.

  

  

   textSource: A System.Windows.Media.TextFormatting.TextSource value that represents the text source for the 

    line.

  

   firstCharIndex: An System.Int32 value that specifies the character index of the starting character in the line.

   paragraphWidth: A System.Double value that specifies the width of the paragraph that the line fills.

   paragraphProperties: A System.Windows.Media.TextFormatting.TextParagraphProperties value that represents paragraph 

    properties,such as flow direction,alignment,or indentation.

  

   previousLineBreak: A System.Windows.Media.TextFormatting.TextLineBreak value that specifies the text formatter 

    state,in terms of where the previous line in the paragraph was broken by the text formatting 

    process.

  

   Returns: A System.Windows.Media.TextFormatting.TextLine value that represents a line of text that can be 

    displayed.
  """
  pass
 def FormatMinMaxParagraphWidth(self,textSource,firstCharIndex,paragraphProperties,textRunCache=None):
  """
  FormatMinMaxParagraphWidth(self: TextFormatter,textSource: TextSource,firstCharIndex: int,paragraphProperties: TextParagraphProperties,textRunCache: TextRunCache) -> MinMaxParagraphWidth

  

   Returns a value that represents the smallest and largest possible paragraph width that can fully 

    contain the specified text content.

  

  

   textSource: A System.Windows.Media.TextFormatting.TextSource object that represents the text source for the 

    line.

  

   firstCharIndex: An System.Int32 value that specifies the character index of the starting character in the line.

   paragraphProperties: A System.Windows.Media.TextFormatting.TextParagraphProperties object that represents paragraph 

    properties,such as flow direction,alignment,or indentation.

  

   textRunCache: A System.Windows.Media.TextFormatting.TextRunCache object that represents the caching mechanism 

    for the layout of text.

  

   Returns: A System.Windows.Media.TextFormatting.MinMaxParagraphWidth value that represents the smallest 

    and largest possible paragraph width that can fully contain the specified text content.

  

  FormatMinMaxParagraphWidth(self: TextFormatter,textSource: TextSource,firstCharIndex: int,paragraphProperties: TextParagraphProperties) -> MinMaxParagraphWidth

  

   Returns a value that represents the smallest and largest possible paragraph width that can fully 

    contain the specified text content.

  

  

   textSource: A System.Windows.Media.TextFormatting.TextSource object that represents the text source for the 

    line.

  

   firstCharIndex: An System.Int32 value that specifies the character index of the starting character in the line.

   paragraphProperties: A System.Windows.Media.TextFormatting.TextParagraphProperties object that represents paragraph 

    properties,such as flow direction,alignment,or indentation.

  

   Returns: A System.Windows.Media.TextFormatting.MinMaxParagraphWidth value that represents the smallest 

    and largest possible paragraph width that can fully contain the specified text content.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass

class TextHidden(TextRun):
 """
 Defines a specialized text run that is used to mark a range of hidden characters.

 

 TextHidden(length: int)
 """
 @staticmethod
 def __new__(self,length):
  """ __new__(cls: type,length: int) """
  pass
 CharacterBufferReference=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a reference to the System.Windows.Media.TextFormatting.TextHidden character buffer.



Get: CharacterBufferReference(self: TextHidden) -> CharacterBufferReference



"""

 Length=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the character length of the System.Windows.Media.TextFormatting.TextHidden character buffer.



Get: Length(self: TextHidden) -> int



"""

 Properties=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the set of properties shared by every text character of the System.Windows.Media.TextFormatting.TextHidden character buffer.



Get: Properties(self: TextHidden) -> TextRunProperties



"""



class TextLine(object,ITextMetrics,IDisposable):
 """ Provides an abstract class for supporting formatting services to a line of text. """
 def Collapse(self,collapsingPropertiesList):
  """
  Collapse(self: TextLine,*collapsingPropertiesList: Array[TextCollapsingProperties]) -> TextLine

  

   Create a collapsed line based on collapsed text properties.

  

   collapsingPropertiesList: A list of System.Windows.Media.TextFormatting.TextCollapsingProperties objects that represent 

    the collapsed text properties.

  

   Returns: A System.Windows.Media.TextFormatting.TextLine value that represents a collapsed line that can 

    be displayed.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: TextLine)

   Releases all managed and unmanaged resources used by the 

    System.Windows.Media.TextFormatting.TextFormatter object.
  """
  pass
 def Draw(self,drawingContext,origin,inversion):
  """
  Draw(self: TextLine,drawingContext: DrawingContext,origin: Point,inversion: InvertAxes)

   Renders the System.Windows.Media.TextFormatting.TextLine object based upon the specified 

    System.Windows.Media.DrawingContext.

  

  

   drawingContext: The System.Windows.Media.DrawingContext object onto which the 

    System.Windows.Media.TextFormatting.TextLine is rendered.

  

   origin: A System.Windows.Point value that represents the drawing origin.

   inversion: An enumerated System.Windows.Media.TextFormatting.InvertAxes value that indicates the inversion 

    of the horizontal and vertical axes of the drawing surface.
  """
  pass
 def GetBackspaceCaretCharacterHit(self,characterHit):
  """
  GetBackspaceCaretCharacterHit(self: TextLine,characterHit: CharacterHit) -> CharacterHit

  

   Gets the previous character hit after backspacing.

  

   characterHit: The current System.Windows.Media.TextFormatting.CharacterHit object.

   Returns: The System.Windows.Media.TextFormatting.CharacterHit object after backspacing.
  """
  pass
 def GetCharacterHitFromDistance(self,distance):
  """
  GetCharacterHitFromDistance(self: TextLine,distance: float) -> CharacterHit

  

   Gets the character hit corresponding to the specified distance from the beginning of the line.

  

   distance: A System.Double value that represents the distance from the beginning of the line.

   Returns: The System.Windows.Media.TextFormatting.CharacterHit object at the specified distance from the 

    beginning of the line.
  """
  pass
 def GetDistanceFromCharacterHit(self,characterHit):
  """
  GetDistanceFromCharacterHit(self: TextLine,characterHit: CharacterHit) -> float

  

   Gets the distance from the beginning of the line to the specified character hit.

  

   characterHit: The System.Windows.Media.TextFormatting.CharacterHit object whose distance you want to query.

   Returns: A System.Double that represents the distance from the beginning of the line.
  """
  pass
 def GetIndexedGlyphRuns(self):
  """
  GetIndexedGlyphRuns(self: TextLine) -> IEnumerable[IndexedGlyphRun]

  

   Gets an enumerator for enumerating System.Windows.Media.TextFormatting.IndexedGlyphRun objects 

    in the System.Windows.Media.TextFormatting.TextLine.

  

   Returns: An enumerator that allows you to enumerate each 

    System.Windows.Media.TextFormatting.IndexedGlyphRun object in the 

    System.Windows.Media.TextFormatting.TextLine.
  """
  pass
 def GetNextCaretCharacterHit(self,characterHit):
  """
  GetNextCaretCharacterHit(self: TextLine,characterHit: CharacterHit) -> CharacterHit

  

   Gets the next character hit for caret navigation.

  

   characterHit: The current System.Windows.Media.TextFormatting.CharacterHit object.

   Returns: The next System.Windows.Media.TextFormatting.CharacterHit object based on caret navigation.
  """
  pass
 def GetPreviousCaretCharacterHit(self,characterHit):
  """
  GetPreviousCaretCharacterHit(self: TextLine,characterHit: CharacterHit) -> CharacterHit

  

   Gets the previous character hit for caret navigation.

  

   characterHit: The current System.Windows.Media.TextFormatting.CharacterHit object.

   Returns: The previous System.Windows.Media.TextFormatting.CharacterHit object based on caret navigation.
  """
  pass
 def GetTextBounds(self,firstTextSourceCharacterIndex,textLength):
  """
  GetTextBounds(self: TextLine,firstTextSourceCharacterIndex: int,textLength: int) -> IList[TextBounds]

  

   Gets an array of bounding rectangles that represent the range of characters within a text line.

  

   firstTextSourceCharacterIndex: An System.Int32 value that represents the index of first character of specified range.

   textLength: An System.Int32 value that represents the number of characters of the specified range.

   Returns: A list of System.Windows.Media.TextFormatting.TextBounds objects representing the bounding 

    rectangle.
  """
  pass
 def GetTextCollapsedRanges(self):
  """
  GetTextCollapsedRanges(self: TextLine) -> IList[TextCollapsedRange]

  

   Gets a collection of collapsed text ranges after a line has been collapsed.

   Returns: A list of System.Windows.Media.TextFormatting.TextCollapsedRange objects that represent the 

    collapsed text.
  """
  pass
 def GetTextLineBreak(self):
  """
  GetTextLineBreak(self: TextLine) -> TextLineBreak

  

   Gets the state of the line when broken by line breaking process.

   Returns: A System.Windows.Media.TextFormatting.TextLineBreak value that represents the line break.
  """
  pass
 def GetTextRunSpans(self):
  """
  GetTextRunSpans(self: TextLine) -> IList[TextSpan[TextRun]]

  

   Gets a collection of System.Windows.Media.TextFormatting.TextRun objects in a text span that are 

    contained within a line.

  

   Returns: Gets a list of System.Windows.Media.TextFormatting.TextRun objects contained within a text span.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*args): #cannot find CLR constructor
  """
  __new__(cls: type,pixelsPerDip: float)

  __new__(cls: type)
  """
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 Baseline=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the distance from the top to the baseline of the current System.Windows.Media.TextFormatting.TextLine object.



Get: Baseline(self: TextLine) -> float



"""

 DependentLength=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of characters following the last character of the line that may trigger reformatting of the current line.



Get: DependentLength(self: TextLine) -> int



"""

 Extent=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the distance from the top-most to bottom-most black pixel in a line.



Get: Extent(self: TextLine) -> float



"""

 HasCollapsed=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the line is collapsed.



Get: HasCollapsed(self: TextLine) -> bool



"""

 HasOverflowed=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether content of the line overflows the specified paragraph width.



Get: HasOverflowed(self: TextLine) -> bool



"""

 Height=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the height of a line of text.



Get: Height(self: TextLine) -> float



"""

 IsTruncated=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Determines whether the text line is truncated in the middle of a word.



Get: IsTruncated(self: TextLine) -> bool



"""

 Length=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the total number of System.Windows.Media.TextFormatting.TextSource positions of the current line.



Get: Length(self: TextLine) -> int



"""

 MarkerBaseline=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the distance from the edge of the line's highest point to the baseline marker of the line.



Get: MarkerBaseline(self: TextLine) -> float



"""

 MarkerHeight=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the height of a marker for a list item.



Get: MarkerHeight(self: TextLine) -> float



"""

 NewlineLength=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of newline characters at the end of a line.



Get: NewlineLength(self: TextLine) -> int



"""

 OverhangAfter=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the distance that black pixels extend beyond the bottom alignment edge of a line.



Get: OverhangAfter(self: TextLine) -> float



"""

 OverhangLeading=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the distance that black pixels extend prior to the left leading alignment edge of the line.



Get: OverhangLeading(self: TextLine) -> float



"""

 OverhangTrailing=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the distance that black pixels extend following the right trailing alignment edge of the line.



Get: OverhangTrailing(self: TextLine) -> float



"""

 PixelsPerDip=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Get: PixelsPerDip(self: TextLine) -> float



Set: PixelsPerDip(self: TextLine)=value

"""

 Start=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the distance from the start of a paragraph to the starting point of a line.



Get: Start(self: TextLine) -> float



"""

 TextBaseline=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the distance from the top to the baseline of the line of text.



Get: TextBaseline(self: TextLine) -> float



"""

 TextHeight=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the height of the text and any other content in the line.



Get: TextHeight(self: TextLine) -> float



"""

 TrailingWhitespaceLength=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of whitespace code points beyond the last non-blank character in a line.



Get: TrailingWhitespaceLength(self: TextLine) -> int



"""

 Width=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the width of a line of text,excluding trailing whitespace characters.



Get: Width(self: TextLine) -> float



"""

 WidthIncludingTrailingWhitespace=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the width of a line of text,including trailing whitespace characters.



Get: WidthIncludingTrailingWhitespace(self: TextLine) -> float



"""



class TextLineBreak(object,IDisposable):
 """ Specifies text properties and state at the point where text is broken by the line breaking process. """
 def Clone(self):
  """
  Clone(self: TextLineBreak) -> TextLineBreak

  

   Clone a new instance of the System.Windows.Media.TextFormatting.TextLineBreak object.

   Returns: A new instance of System.Windows.Media.TextFormatting.TextLineBreak.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: TextLineBreak)

   Releases the resources used by the System.Windows.Media.TextFormatting.TextLineBreak class.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass

class TextMarkerProperties(object):
 """ Represents an abstract class for defining text markers. """
 Offset=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the distance from the start of the line to the end of the text marker symbol.



Get: Offset(self: TextMarkerProperties) -> float



"""

 TextSource=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Media.TextFormatting.TextSource that represents the source of the text runs for the marker symbol.



Get: TextSource(self: TextMarkerProperties) -> TextSource



"""



class TextModifier(TextRun):
 """ Represents a specialized text run that can be used to modify properties of text runs within its scope. """
 def ModifyProperties(self,properties):
  """
  ModifyProperties(self: TextModifier,properties: TextRunProperties) -> TextRunProperties

  

   Retrieves the System.Windows.Media.TextFormatting.TextRunProperties for a text run.

  

   properties: The System.Windows.Media.TextFormatting.TextRunProperties for a text run,or the return value of 

    System.Windows.Media.TextFormatting.TextModifier.ModifyProperties(System.Windows.Media.TextFormat

    ting.TextRunProperties) for a nested text modifier.

  

   Returns: The actual System.Windows.Media.TextFormatting.TextRunProperties to be used by the 

    System.Windows.Media.TextFormatting.TextFormatter,subject to further modification by 

    System.Windows.Media.TextFormatting.TextModifier objects at outer scopes.
  """
  pass
 CharacterBufferReference=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Media.TextFormatting.CharacterBufferReference for the System.Windows.Media.TextFormatting.TextModifier.



Get: CharacterBufferReference(self: TextModifier) -> CharacterBufferReference



"""

 FlowDirection=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.FlowDirection for the System.Windows.Media.TextFormatting.TextModifier.



Get: FlowDirection(self: TextModifier) -> FlowDirection



"""

 HasDirectionalEmbedding=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the System.Windows.Media.TextFormatting.TextModifier supports System.Windows.FlowDirection for the current scope of text.



Get: HasDirectionalEmbedding(self: TextModifier) -> bool



"""



class TextParagraphProperties(object):
 """ Provides a set of properties,such as flow direction,alignment,or indentation,that can be applied to a paragraph. This is an abstract class. """
 AlwaysCollapsible=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a formatted line can always be collapsed.



Get: AlwaysCollapsible(self: TextParagraphProperties) -> bool



"""

 DefaultIncrementalTab=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the default incremental tab distance.



Get: DefaultIncrementalTab(self: TextParagraphProperties) -> float



"""

 DefaultTextRunProperties=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the default text run properties,such as typeface or foreground brush.



Get: DefaultTextRunProperties(self: TextParagraphProperties) -> TextRunProperties



"""

 FirstLineInParagraph=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the text run is the first line of the paragraph.



Get: FirstLineInParagraph(self: TextParagraphProperties) -> bool



"""

 FlowDirection=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that specifies whether the primary text advance direction shall be left-to-right,or right-to-left.



Get: FlowDirection(self: TextParagraphProperties) -> FlowDirection



"""

 Indent=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the amount of line indentation.



Get: Indent(self: TextParagraphProperties) -> float



"""

 LineHeight=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the height of a line of text.



Get: LineHeight(self: TextParagraphProperties) -> float



"""

 ParagraphIndent=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the amount of the paragraph indentation.



Get: ParagraphIndent(self: TextParagraphProperties) -> float



"""

 Tabs=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a collection of tab definitions.



Get: Tabs(self: TextParagraphProperties) -> IList[TextTabProperties]



"""

 TextAlignment=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that describes how an inline content of a block is aligned.



Get: TextAlignment(self: TextParagraphProperties) -> TextAlignment



"""

 TextDecorations=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the collection of System.Windows.TextDecoration objects.



Get: TextDecorations(self: TextParagraphProperties) -> TextDecorationCollection



"""

 TextMarkerProperties=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that specifies marker characteristics of the first line in the paragraph.



Get: TextMarkerProperties(self: TextParagraphProperties) -> TextMarkerProperties



"""

 TextWrapping=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that controls whether text wraps when it reaches the flow edge of its containing block box.



Get: TextWrapping(self: TextParagraphProperties) -> TextWrapping



"""



class TextRunBounds(object):
 """ Represents the bounding rectangle of a text run. """
 Length=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the character length of bounded text run.



Get: Length(self: TextRunBounds) -> int



"""

 Rectangle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the bounding rectangle for the text run.



Get: Rectangle(self: TextRunBounds) -> Rect



"""

 TextRun=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Media.TextFormatting.TextRun object that represents the text run.



Get: TextRun(self: TextRunBounds) -> TextRun



"""

 TextSourceCharacterIndex=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the character index of the first character in the bounded text run.



Get: TextSourceCharacterIndex(self: TextRunBounds) -> int



"""



class TextRunCache(object):
 """
 Provides caching services to the System.Windows.Media.TextFormatting.TextFormatter object in order to improve performance.

 

 TextRunCache()
 """
 def Change(self,textSourceCharacterIndex,addition,removal):
  """
  Change(self: TextRunCache,textSourceCharacterIndex: int,addition: int,removal: int)

   Notifies the text engine client of a change to the cache when text content or text run 

    properties of System.Windows.Media.TextFormatting.TextRun are added,removed,or replaced.

  

  

   textSourceCharacterIndex: Specifies the System.Windows.Media.TextFormatting.TextSource character index position of the 

    start of the change.

  

   addition: Indicates the number of System.Windows.Media.TextFormatting.TextSource characters to be added.

   removal: Indicates the number of System.Windows.Media.TextFormatting.TextSource characters to be removed.
  """
  pass
 def Invalidate(self):
  """
  Invalidate(self: TextRunCache)

   Signals the text engine client to invalidate the entire contents of the 

    System.Windows.Media.TextFormatting.TextFormatter cache.
  """
  pass

class TextRunProperties(object):
 """ Provides a set of properties,such as typeface or foreground brush,that can be applied to a System.Windows.Media.TextFormatting.TextRun object. This is an abstract class. """
 BackgroundBrush=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the brush that is used to paint the background color of the text run.



Get: BackgroundBrush(self: TextRunProperties) -> Brush



"""

 BaselineAlignment=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the baseline style for a text that is positioned on the vertical axis.



Get: BaselineAlignment(self: TextRunProperties) -> BaselineAlignment



"""

 CultureInfo=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the culture information for the text run.



Get: CultureInfo(self: TextRunProperties) -> CultureInfo



"""

 FontHintingEmSize=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the text size in points,which is then used for font hinting.



Get: FontHintingEmSize(self: TextRunProperties) -> float



"""

 FontRenderingEmSize=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the text size in points for the text run.



Get: FontRenderingEmSize(self: TextRunProperties) -> float



"""

 ForegroundBrush=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the brush that is used to paint the foreground color of the text run.



Get: ForegroundBrush(self: TextRunProperties) -> Brush



"""

 NumberSubstitution=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number substitution settings,which determines who numbers in text are displayed in different cultures.



Get: NumberSubstitution(self: TextRunProperties) -> NumberSubstitution



"""

 PixelsPerDip=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Get: PixelsPerDip(self: TextRunProperties) -> float



Set: PixelsPerDip(self: TextRunProperties)=value

"""

 TextDecorations=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the collection of  System.Windows.TextDecoration objects used for the text run.



Get: TextDecorations(self: TextRunProperties) -> TextDecorationCollection



"""

 TextEffects=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the collection of System.Windows.Media.TextEffect objects used for the text run.



Get: TextEffects(self: TextRunProperties) -> TextEffectCollection



"""

 Typeface=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the typeface for the text run.



Get: Typeface(self: TextRunProperties) -> Typeface



"""

 TypographyProperties=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the typography properties for the text run.



Get: TypographyProperties(self: TextRunProperties) -> TextRunTypographyProperties



"""



class TextRunTypographyProperties(object):
 """ Provides an abstract class for supporting typography properties for System.Windows.Media.TextFormatting.TextRun objects. """
 def OnPropertiesChanged(self,*args):
  """
  OnPropertiesChanged(self: TextRunTypographyProperties)

   Corrects internal state for a System.Windows.Media.TextFormatting.TextRunTypographyProperties 

    derived class whenever any System.Windows.Media.TextFormatting.TextRunTypographyProperties 

    property changes its value.
  """
  pass
 AnnotationAlternates=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the index of an alternate annotation form.



Get: AnnotationAlternates(self: TextRunTypographyProperties) -> int



"""

 Capitals=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates the capital form of the selected font.



Get: Capitals(self: TextRunTypographyProperties) -> FontCapitals



"""

 CapitalSpacing=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether inter-glyph spacing for all-capital text is globally adjusted to improve readability.



Get: CapitalSpacing(self: TextRunTypographyProperties) -> bool



"""

 CaseSensitiveForms=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether glyphs adjust their vertical position to better align with uppercase glyphs.



Get: CaseSensitiveForms(self: TextRunTypographyProperties) -> bool



"""

 ContextualAlternates=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether custom glyph forms can be used based upon the context of the text being rendered.



Get: ContextualAlternates(self: TextRunTypographyProperties) -> bool



"""

 ContextualLigatures=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether contextual ligatures are enabled.



Get: ContextualLigatures(self: TextRunTypographyProperties) -> bool



"""

 ContextualSwashes=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that specifies the index of a contextual swashes form.



Get: ContextualSwashes(self: TextRunTypographyProperties) -> int



"""

 DiscretionaryLigatures=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether discretionary ligatures are enabled.



Get: DiscretionaryLigatures(self: TextRunTypographyProperties) -> bool



"""

 EastAsianExpertForms=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the standard Japanese font forms have been replaced with the corresponding preferred typographic forms.



Get: EastAsianExpertForms(self: TextRunTypographyProperties) -> bool



"""

 EastAsianLanguage=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates the version of glyphs to be used for a specific writing system or language.



Get: EastAsianLanguage(self: TextRunTypographyProperties) -> FontEastAsianLanguage



"""

 EastAsianWidths=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates the proportional width to be used for Latin characters in an East Asian font.



Get: EastAsianWidths(self: TextRunTypographyProperties) -> FontEastAsianWidths



"""

 Fraction=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates the fraction style.



Get: Fraction(self: TextRunTypographyProperties) -> FontFraction



"""

 HistoricalForms=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether historical forms are enabled.



Get: HistoricalForms(self: TextRunTypographyProperties) -> bool



"""

 HistoricalLigatures=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether historical ligatures are enabled.



Get: HistoricalLigatures(self: TextRunTypographyProperties) -> bool



"""

 Kerning=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether kerning is enabled.



Get: Kerning(self: TextRunTypographyProperties) -> bool



"""

 MathematicalGreek=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether standard typographic font forms of Greek glyphs have been replaced with corresponding font forms commonly used in mathematical notation.



Get: MathematicalGreek(self: TextRunTypographyProperties) -> bool



"""

 NumeralAlignment=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the alignment of widths when using numerals.



Get: NumeralAlignment(self: TextRunTypographyProperties) -> FontNumeralAlignment



"""

 NumeralStyle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates the set of glyphs that are used to render numeric alternate font forms.



Get: NumeralStyle(self: TextRunTypographyProperties) -> FontNumeralStyle



"""

 SlashedZero=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a nominal zero font form should be replaced with a slashed zero.



Get: SlashedZero(self: TextRunTypographyProperties) -> bool



"""

 StandardLigatures=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether standard ligatures are enabled.



Get: StandardLigatures(self: TextRunTypographyProperties) -> bool



"""

 StandardSwashes=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the index of a standard swashes form.



Get: StandardSwashes(self: TextRunTypographyProperties) -> int



"""

 StylisticAlternates=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the index of a stylistic alternates form.



Get: StylisticAlternates(self: TextRunTypographyProperties) -> int



"""

 StylisticSet1=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a stylistic set of a font form is enabled.



Get: StylisticSet1(self: TextRunTypographyProperties) -> bool



"""

 StylisticSet10=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a stylistic set of a font form is enabled.



Get: StylisticSet10(self: TextRunTypographyProperties) -> bool



"""

 StylisticSet11=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a stylistic set of a font form is enabled.



Get: StylisticSet11(self: TextRunTypographyProperties) -> bool



"""

 StylisticSet12=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a stylistic set of a font form is enabled.



Get: StylisticSet12(self: TextRunTypographyProperties) -> bool



"""

 StylisticSet13=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a stylistic set of a font form is enabled.



Get: StylisticSet13(self: TextRunTypographyProperties) -> bool



"""

 StylisticSet14=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a stylistic set of a font form is enabled.



Get: StylisticSet14(self: TextRunTypographyProperties) -> bool



"""

 StylisticSet15=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a stylistic set of a font form is enabled.



Get: StylisticSet15(self: TextRunTypographyProperties) -> bool



"""

 StylisticSet16=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a stylistic set of a font form is enabled.



Get: StylisticSet16(self: TextRunTypographyProperties) -> bool



"""

 StylisticSet17=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a stylistic set of a font form is enabled.



Get: StylisticSet17(self: TextRunTypographyProperties) -> bool



"""

 StylisticSet18=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a stylistic set of a font form is enabled.



Get: StylisticSet18(self: TextRunTypographyProperties) -> bool



"""

 StylisticSet19=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a stylistic set of a font form is enabled.



Get: StylisticSet19(self: TextRunTypographyProperties) -> bool



"""

 StylisticSet2=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a stylistic set of a font form is enabled.



Get: StylisticSet2(self: TextRunTypographyProperties) -> bool



"""

 StylisticSet20=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a stylistic set of a font form is enabled.



Get: StylisticSet20(self: TextRunTypographyProperties) -> bool



"""

 StylisticSet3=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a stylistic set of a font form is enabled.



Get: StylisticSet3(self: TextRunTypographyProperties) -> bool



"""

 StylisticSet4=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a stylistic set of a font form is enabled.



Get: StylisticSet4(self: TextRunTypographyProperties) -> bool



"""

 StylisticSet5=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a stylistic set of a font form is enabled.



Get: StylisticSet5(self: TextRunTypographyProperties) -> bool



"""

 StylisticSet6=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a stylistic set of a font form is enabled.



Get: StylisticSet6(self: TextRunTypographyProperties) -> bool



"""

 StylisticSet7=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a stylistic set of a font form is enabled.



Get: StylisticSet7(self: TextRunTypographyProperties) -> bool



"""

 StylisticSet8=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a stylistic set of a font form is enabled.



Get: StylisticSet8(self: TextRunTypographyProperties) -> bool



"""

 StylisticSet9=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a stylistic set of a font form is enabled.



Get: StylisticSet9(self: TextRunTypographyProperties) -> bool



"""

 Variants=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates a variation of the standard typographic form to be used.



Get: Variants(self: TextRunTypographyProperties) -> FontVariants



"""



class TextSimpleMarkerProperties(TextMarkerProperties):
 """
 Provides for a generic implementation of text marker properties.

 

 TextSimpleMarkerProperties(style: TextMarkerStyle,offset: float,autoNumberingIndex: int,textParagraphProperties: TextParagraphProperties)
 """
 @staticmethod
 def __new__(self,style,offset,autoNumberingIndex,textParagraphProperties):
  """ __new__(cls: type,style: TextMarkerStyle,offset: float,autoNumberingIndex: int,textParagraphProperties: TextParagraphProperties) """
  pass
 Offset=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the distance from the start of the line to the end of the text marker symbol.



Get: Offset(self: TextSimpleMarkerProperties) -> float



"""

 TextSource=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the source of the text runs used for the text marker.



Get: TextSource(self: TextSimpleMarkerProperties) -> TextSource



"""



class TextSource(object):
 """ Provides an abstract class for specifying character data and formatting properties to be used by the System.Windows.Media.TextFormatting.TextFormatter object. """
 def GetPrecedingText(self,textSourceCharacterIndexLimit):
  """
  GetPrecedingText(self: TextSource,textSourceCharacterIndexLimit: int) -> TextSpan[CultureSpecificCharacterBufferRange]

  

   Retrieves the text span immediately before the specified 

    System.Windows.Media.TextFormatting.TextSource position.

  

  

   textSourceCharacterIndexLimit: An System.Int32 value that specifies the character index position where text retrieval stops.

   Returns: A System.Windows.Media.TextFormatting.CultureSpecificCharacterBufferRange value that represents 

    the text span immediately before textSourceCharacterIndexLimit.
  """
  pass
 def GetTextEffectCharacterIndexFromTextSourceCharacterIndex(self,textSourceCharacterIndex):
  """
  GetTextEffectCharacterIndexFromTextSourceCharacterIndex(self: TextSource,textSourceCharacterIndex: int) -> int

  

   Retrieves a value that maps a System.Windows.Media.TextFormatting.TextSource character index to 

    a System.Windows.Media.TextEffect character index.

  

  

   textSourceCharacterIndex: An System.Int32 value that specifies the System.Windows.Media.TextFormatting.TextSource 

    character index to map.

  

   Returns: An System.Int32 value that represents the System.Windows.Media.TextEffect character index.
  """
  pass
 def GetTextRun(self,textSourceCharacterIndex):
  """
  GetTextRun(self: TextSource,textSourceCharacterIndex: int) -> TextRun

  

   Retrieves a System.Windows.Media.TextFormatting.TextRun starting at a specified 

    System.Windows.Media.TextFormatting.TextSource position.

  

  

   textSourceCharacterIndex: Specifies the character index position in the System.Windows.Media.TextFormatting.TextSource 

    where the System.Windows.Media.TextFormatting.TextRun is retrieved.

  

   Returns: A value that represents a System.Windows.Media.TextFormatting.TextRun,or an object derived from 

    System.Windows.Media.TextFormatting.TextRun.
  """
  pass
 PixelsPerDip=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Get: PixelsPerDip(self: TextSource) -> float



Set: PixelsPerDip(self: TextSource)=value

"""



class TextSpan(object):
 """ TextSpan[T](length: int,value: T) """
 @staticmethod
 def __new__(self,length,value):
  """ __new__(cls: type,length: int,value: T) """
  pass
 Length=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of characters in the text span.



Get: Length(self: TextSpan[T]) -> int



"""

 Value=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the object associated with the text span.



Get: Value(self: TextSpan[T]) -> T



"""



class TextTabAlignment(Enum,IComparable,IFormattable,IConvertible):
 """
 Describes how text is aligned at a tab location.

 

 enum TextTabAlignment,values: Center (1),Character (3),Left (0),Right (2)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Center=None
 Character=None
 Left=None
 Right=None
 value__=None


class TextTabProperties(object):
 """
 Specifies properties of user-defined tabs.

 

 TextTabProperties(alignment: TextTabAlignment,location: float,tabLeader: int,aligningChar: int)
 """
 @staticmethod
 def __new__(self,alignment,location,tabLeader,aligningChar):
  """ __new__(cls: type,alignment: TextTabAlignment,location: float,tabLeader: int,aligningChar: int) """
  pass
 AligningCharacter=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the index of the specific character in the text that is aligned at the specified tab location.



Get: AligningCharacter(self: TextTabProperties) -> int



"""

 Alignment=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the alignment style of the text at the tab location.



Get: Alignment(self: TextTabProperties) -> TextTabAlignment



"""

 Location=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the index value of the tab location.



Get: Location(self: TextTabProperties) -> float



"""

 TabLeader=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the index of the character that is used to display the tab leader.



Get: TabLeader(self: TextTabProperties) -> int



"""



class TextTrailingCharacterEllipsis(TextCollapsingProperties):
 """
 Defines collapsed text properties for collapsing a whole line toward the end at character granularity,and with ellipsis being the collapsed text symbol.

 

 TextTrailingCharacterEllipsis(width: float,textRunProperties: TextRunProperties)
 """
 @staticmethod
 def __new__(self,width,textRunProperties):
  """ __new__(cls: type,width: float,textRunProperties: TextRunProperties) """
  pass
 Style=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the style of collapsed text.



Get: Style(self: TextTrailingCharacterEllipsis) -> TextCollapsingStyle



"""

 Symbol=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the text run that is used as the collapsed text symbol.



Get: Symbol(self: TextTrailingCharacterEllipsis) -> TextRun



"""

 Width=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the width for which the specified collapsed text range is constrained to.



Get: Width(self: TextTrailingCharacterEllipsis) -> float



"""



class TextTrailingWordEllipsis(TextCollapsingProperties):
 """
 Defines collapsed text properties for collapsing a whole line toward the end at word granularity,and with ellipsis being the collapsed text symbol.

 

 TextTrailingWordEllipsis(width: float,textRunProperties: TextRunProperties)
 """
 @staticmethod
 def __new__(self,width,textRunProperties):
  """ __new__(cls: type,width: float,textRunProperties: TextRunProperties) """
  pass
 Style=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the style of collapsed text.



Get: Style(self: TextTrailingWordEllipsis) -> TextCollapsingStyle



"""

 Symbol=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the text run that is used as the collapsed text symbol.



Get: Symbol(self: TextTrailingWordEllipsis) -> TextRun



"""

 Width=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the width for which the specified collapsed text range is constrained to.



Get: Width(self: TextTrailingWordEllipsis) -> float



"""






"""Provide access to DotNet Framework.

Examples:
    ```python
    from pyrevit.framework import Assembly, Windows
    ```
"""

#pylint: disable=W0703,C0302,C0103,W0614,E0401,W0611,C0413,ungrouped-imports
import os.path as op
from pyrevit.compat import IRONPY, NETCORE, PY2

import clr
import System

# netcore init
if NETCORE:
    clr.AddReference('System.Runtime')
    clr.AddReference('System.Text.RegularExpressions')
    clr.AddReference('System.Diagnostics.Process')
    clr.AddReference('System.IO.FileSystem.DriveInfo')
    clr.AddReference('System.Net.WebClient')
    clr.AddReference('System.Net.Requests')
    clr.AddReference('System.Net.WebProxy')
    clr.AddReference('System.Runtime.Serialization.Formatters')
    clr.AddReference('System.Reflection.Emit')
    clr.AddReference('Lokad.ILPack')
    clr.AddReference('System.ComponentModel')
    clr.AddReference('System.ObjectModel')
    clr.AddReference('System.Diagnostics.FileVersionInfo')

clr.AddReference('System.Core')
clr.AddReference('System.Management')
clr.AddReference('System.Windows.Forms')
clr.AddReference('System.Drawing')
clr.AddReference('PresentationCore')
clr.AddReference('PresentationFramework')
clr.AddReference('System.Xml.Linq')
clr.AddReference('WindowsBase')

from System import AppDomain, Version
from System import Type
from System import Uri, UriKind, Guid
from System import EventHandler
from System import Array, IntPtr, Enum, Byte
from System import Convert
from System.Text import Encoding
from System.Text.RegularExpressions import Regex
from System.Collections import ObjectModel
from System.Collections.ObjectModel import ObservableCollection
from System.Collections import IEnumerator, IEnumerable
from System.Collections.Generic import List, Dictionary
from System.Collections.Generic import IList, IDictionary
from System.Collections.Generic import KeyValuePair
from System import DateTime, DateTimeOffset

from System import Diagnostics
from System.Diagnostics import Process
from System.Diagnostics import Stopwatch

from System import Reflection
from System.Reflection import Assembly, AssemblyName
from System.Reflection import TypeAttributes, MethodAttributes
from System.Reflection import CallingConventions
from System.Reflection import BindingFlags
from System.Reflection.Emit import AssemblyBuilderAccess
from System.Reflection.Emit import CustomAttributeBuilder, OpCodes

from System import IO
from System.IO import IOException, DriveInfo, Path, StringReader, File

from System import Net
from System.Net import WebClient, WebRequest, WebProxy

from System import ComponentModel
from System import Drawing
from System import Windows
from System.Windows import Forms
from System.Windows.Forms import Clipboard
from System.Windows import Controls
from System.Windows import Documents
from System.Windows import Media
from System.Windows import Threading
from System.Windows import Interop
from System.Windows import Input
from System.Windows import Data
from System.Windows import ResourceDictionary
from System.Windows.Media import Imaging, SolidColorBrush, Color
from System import Math
from System.Management import ManagementObjectSearcher
from System.Runtime.Serialization import FormatterServices
from System.Linq import Enumerable

import pyrevit.engine as eng


ASSEMBLY_FILE_TYPE = 'dll'
ASSEMBLY_FILE_EXT = '.dll'


ipy_assmname = '{prefix}IronPython'.format(prefix=eng.EnginePrefix)
ipy_dllpath = op.join(eng.EnginePath, ipy_assmname + ASSEMBLY_FILE_EXT)
if IRONPY:
    clr.AddReferenceToFileAndPath(ipy_dllpath)
else:
    clr.AddReference(ipy_assmname)

import IronPython

# WPF
wpf = None
if IRONPY:
    wpf_assmname = '{prefix}IronPython.Wpf'.format(prefix=eng.EnginePrefix)
    wpf_dllpath = op.join(eng.EnginePath, wpf_assmname + ASSEMBLY_FILE_EXT)
    clr.AddReferenceToFileAndPath(wpf_dllpath)
    import wpf


# SQLite
sqlite3 = None
sqlite3_assmname = '{prefix}IronPython.SQLite'.format(prefix=eng.EnginePrefix)
sqlite3_dllpath = op.join(eng.EnginePath, sqlite3_assmname + ASSEMBLY_FILE_EXT)
if IRONPY:
    clr.AddReferenceToFileAndPath(sqlite3_dllpath)
else:
    clr.AddReference(sqlite3_dllpath)
import sqlite3



CPDialogs = None
try:
    clr.AddReference('Microsoft.WindowsAPICodePack')
    clr.AddReference('Microsoft.WindowsAPICodePack.Shell')
    import Microsoft.WindowsAPICodePack.Dialogs as CPDialogs #pylint: disable=ungrouped-imports
except Exception:
    pass


# try loading some utility modules shipped with revit
NSJson = None
try:
    clr.AddReference('pyRevitLabs.Json')
    import pyRevitLabs.Json as NSJson
except Exception:
    pass

clr.AddReference('pyRevitLabs.Emojis')
import pyRevitLabs.Emojis as Emojis


# do not import anything from pyrevit before this
from pyrevit import BIN_DIR


def get_type(fw_object):
    """Return CLR type of an object."""
    return clr.GetClrType(fw_object)


def get_dll_file(assembly_name):
    """Return path to given assembly name."""
    addin_file = op.join(BIN_DIR, assembly_name + '.dll')
    if op.exists(addin_file):
        return addin_file


def get_current_thread_id():
    """Return manageed thread id of current thread."""
    return System.Threading.Thread.CurrentThread.ManagedThreadId



class HttpListenerPrefixCollection(object,ICollection[str],IEnumerable[str],IEnumerable):
 """ Represents the collection used to store Uniform Resource Identifier (URI) prefixes for System.Net.HttpListener objects. """
 def Add(self,uriPrefix):
  """
  Add(self: HttpListenerPrefixCollection,uriPrefix: str)

   Adds a Uniform Resource Identifier (URI) prefix to the collection.

  

   uriPrefix: A System.String that identifies the URI information that is compared in incoming requests. The 

    prefix must be terminated with a forward slash ("/").
  """
  pass
 def Clear(self):
  """
  Clear(self: HttpListenerPrefixCollection)

   Removes all the Uniform Resource Identifier (URI) prefixes from the collection.
  """
  pass
 def Contains(self,uriPrefix):
  """
  Contains(self: HttpListenerPrefixCollection,uriPrefix: str) -> bool

  

   Returns a System.Boolean value that indicates whether the specified prefix is contained in the 

    collection.

  

  

   uriPrefix: A System.String that contains the Uniform Resource Identifier (URI) prefix to test.

   Returns: true if this collection contains the prefix specified by uriPrefix; otherwise,false.
  """
  pass
 def CopyTo(self,array,offset):
  """
  CopyTo(self: HttpListenerPrefixCollection,array: Array[str],offset: int)

   Copies the contents of an System.Net.HttpListenerPrefixCollection to the specified string array.

  

   array: The one dimensional string array that receives the Uniform Resource Identifier (URI) prefix 

    strings in this collection.

  

   offset: The zero-based index in array at which copying begins.

  CopyTo(self: HttpListenerPrefixCollection,array: Array,offset: int)

   Copies the contents of an System.Net.HttpListenerPrefixCollection to the specified array.

  

   array: The one dimensional System.Array that receives the Uniform Resource Identifier (URI) prefix 

    strings in this collection.

  

   offset: The zero-based index in array at which copying begins.
  """
  pass
 def GetEnumerator(self):
  """
  GetEnumerator(self: HttpListenerPrefixCollection) -> IEnumerator[str]

  

   Returns an object that can be used to iterate through the collection.

   Returns: An object that implements the System.Collections.IEnumerator interface and provides access to 

    the strings in this collection.
  """
  pass
 def Remove(self,uriPrefix):
  """
  Remove(self: HttpListenerPrefixCollection,uriPrefix: str) -> bool

  

   Removes the specified Uniform Resource Identifier (URI) from the list of prefixes handled by the 

    System.Net.HttpListener object.

  

  

   uriPrefix: A System.String that contains the URI prefix to remove.

   Returns: true if the uriPrefix was found in the System.Net.HttpListenerPrefixCollection and removed; 

    otherwise false.
  """
  pass
 def __add__(self,*args):
  """ x.__add__(y) <==> x+y """
  pass
 def __contains__(self,*args):
  """ __contains__(self: ICollection[str],item: str) -> bool """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __iter__(self,*args):
  """ __iter__(self: IEnumerable) -> object """
  pass
 def __len__(self,*args):
  """ x.__len__() <==> len(x) """
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 Count=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of prefixes contained in the collection.



Get: Count(self: HttpListenerPrefixCollection) -> int



"""

 IsReadOnly=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether access to the collection is read-only.



Get: IsReadOnly(self: HttpListenerPrefixCollection) -> bool



"""

 IsSynchronized=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether access to the collection is synchronized (thread-safe).



Get: IsSynchronized(self: HttpListenerPrefixCollection) -> bool



"""





class DataGridViewRowHeightInfoNeededEventArgs(EventArgs):
 """ Provides data for the System.Windows.Forms.DataGridView.RowHeightInfoNeeded event of a System.Windows.Forms.DataGridView. """
 Height=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the height of the row the event occurred for.



Get: Height(self: DataGridViewRowHeightInfoNeededEventArgs) -> int



Set: Height(self: DataGridViewRowHeightInfoNeededEventArgs)=value

"""

 MinimumHeight=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the minimum height of the row the event occurred for.



Get: MinimumHeight(self: DataGridViewRowHeightInfoNeededEventArgs) -> int



Set: MinimumHeight(self: DataGridViewRowHeightInfoNeededEventArgs)=value

"""

 RowIndex=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the index of the row associated with this System.Windows.Forms.DataGridViewRowHeightInfoNeededEventArgs.



Get: RowIndex(self: DataGridViewRowHeightInfoNeededEventArgs) -> int



"""


